@@ -14,6 +14,8 @@
  * This file is dual licensed under LGPL v2.1 and the Scintilla license (http://www.scintilla.org/License.txt).
  */
 
+#include <cmath>
+
 #include <string_view>
 #include <vector>
 
@@ -316,7 +318,7 @@ const CGFloat paddingHighlightY = 2;
 		// main thread). We need that later for idle event processing.
 		NSNotificationCenter *center = [NSNotificationCenter defaultCenter];
 		notificationQueue = [[NSNotificationQueue alloc] initWithNotificationCenter: center];
-		[center addObserver: self selector: @selector(idleTriggered:) name: @"Idle" object: nil];
+		[center addObserver: self selector: @selector(idleTriggered:) name: @"Idle" object: self];
 	}
 	return self;
 }
@@ -818,7 +820,7 @@ namespace {
  */
 
 bool SupportAnimatedFind() {
-	return floor(NSAppKitVersionNumber) < NSAppKitVersionNumber10_12;
+	return std::floor(NSAppKitVersionNumber) < NSAppKitVersionNumber10_12;
 }
 
 }
@@ -1407,7 +1409,7 @@ void ScintillaCocoa::StartDrag() {
 	//           the full rectangle which may include non-selected text.
 
 	NSBitmapImageRep *bitmap = NULL;
-	CGImageRef imagePixmap = pixmap.GetImage();
+	CGImageRef imagePixmap = pixmap.CreateImage();
 	if (imagePixmap)
 		bitmap = [[NSBitmapImageRep alloc] initWithCGImage: imagePixmap];
 	CGImageRelease(imagePixmap);
@@ -2173,26 +2175,39 @@ bool ScintillaCocoa::KeyboardInput(NSEvent *event) {
 /**
  * Used to insert already processed text provided by the Cocoa text input system.
  */
-ptrdiff_t ScintillaCocoa::InsertText(NSString *input) {
-	CFStringEncoding encoding = EncodingFromCharacterSet(IsUnicodeMode(),
-				    vs.styles[STYLE_DEFAULT].characterSet);
-	std::string encoded = EncodedBytesString((__bridge CFStringRef)input, encoding);
+ptrdiff_t ScintillaCocoa::InsertText(NSString *input, CharacterSource charSource) {
+	if ([input length] == 0) {
+		return 0;
+	}
 
-	if (encoded.length() > 0) {
-		if (encoding == kCFStringEncodingUTF8) {
-			// There may be multiple characters in input so loop over them
-			std::string_view sv = encoded;
-			while (sv.length()) {
-				const unsigned char leadByte = sv[0];
-				const unsigned int bytesInCharacter = UTF8BytesOfLead[leadByte];
-				AddCharUTF(sv.data(), bytesInCharacter, false);
-				sv.remove_prefix(bytesInCharacter);
-			}
-		} else {
-			AddCharUTF(encoded.c_str(), static_cast<unsigned int>(encoded.length()), false);
+	// There may be multiple characters in input so loop over them
+	if (IsUnicodeMode()) {
+		// There may be non-BMP characters as 2 elements in NSString so
+		// convert to UTF-8 and use UTF8BytesOfLead.
+		std::string encoded = EncodedBytesString((__bridge CFStringRef)input,
+							 kCFStringEncodingUTF8);
+		std::string_view sv = encoded;
+		while (sv.length()) {
+			const unsigned char leadByte = sv[0];
+			const unsigned int bytesInCharacter = UTF8BytesOfLead[leadByte];
+			InsertCharacter(sv.substr(0, bytesInCharacter), charSource);
+			sv.remove_prefix(bytesInCharacter);
 		}
+		return encoded.length();
+	} else {
+		const CFStringEncoding encoding = EncodingFromCharacterSet(IsUnicodeMode(),
+									   vs.styles[STYLE_DEFAULT].characterSet);
+		ptrdiff_t lengthInserted = 0;
+		for (NSInteger i = 0; i < [input length]; i++) {
+			NSString *character = [input substringWithRange:NSMakeRange(i, 1)];
+			std::string encoded = EncodedBytesString((__bridge CFStringRef)character,
+								 encoding);
+			lengthInserted += encoded.length();
+			InsertCharacter(encoded, charSource);
+		}
+
+		return lengthInserted;
 	}
-	return encoded.length();
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -2272,7 +2287,7 @@ void ScintillaCocoa::CompositionStart() {
  */
 void ScintillaCocoa::CompositionCommit() {
 	pdoc->TentativeCommit();
-	pdoc->DecorationSetCurrentIndicator(INDIC_IME);
+	pdoc->DecorationSetCurrentIndicator(INDICATOR_IME);
 	pdoc->DecorationFillRange(0, 0, pdoc->Length());
 }
 
@@ -2486,7 +2501,7 @@ void ScintillaCocoa::ShowFindIndicatorForRange(NSRange charRange, BOOL retaining
 		layerFindIndicator = [[FindHighlightLayer alloc] init];
 		[content setWantsLayer: YES];
 		layerFindIndicator.geometryFlipped = content.layer.geometryFlipped;
-		if (floor(NSAppKitVersionNumber) > NSAppKitVersionNumber10_8) {
+		if (std::floor(NSAppKitVersionNumber) > NSAppKitVersionNumber10_8) {
 			// Content layer is unflipped on 10.9, but the indicator shows wrong unless flipped
 			layerFindIndicator.geometryFlipped = YES;
 		}
