@@ -9,6 +9,8 @@
  * This file is dual licensed under LGPL v2.1 and the Scintilla license (http://www.scintilla.org/License.txt).
  */
 
+#include <cmath>
+
 #include <string_view>
 #include <vector>
 
@@ -58,7 +60,7 @@ static NSCursor *cursorFromEnum(Window::Cursor cursor) {
 	[super tile];
 
 #if defined(MAC_OS_X_VERSION_10_14)
-	if (floor(NSAppKitVersionNumber) > NSAppKitVersionNumber10_13) {
+	if (std::floor(NSAppKitVersionNumber) > NSAppKitVersionNumber10_13) {
 		NSRect frame = self.contentView.frame;
 		frame.origin.x = self.verticalRulerView.requiredThickness;
 		frame.size.width -= frame.origin.x;
@@ -413,7 +415,7 @@ static NSCursor *cursorFromEnum(Window::Cursor cursor) {
 	}
 
 	[mOwner message: SCI_SETTARGETRANGE wParam: posRange.location lParam: NSMaxRange(posRange)];
-	std::string text([mOwner message: SCI_TARGETASUTF8] + 1, 0);
+	std::string text([mOwner message: SCI_TARGETASUTF8], 0);
 	[mOwner message: SCI_TARGETASUTF8 wParam: 0 lParam: reinterpret_cast<sptr_t>(&text[0])];
 	text = FixInvalidUTF8(text);
 	NSString *result = @(text.c_str());
@@ -423,12 +425,14 @@ static NSCursor *cursorFromEnum(Window::Cursor cursor) {
 	// SCI_GETSTYLEAT reports a signed byte but want an unsigned to index into styles
 	const char styleByte = static_cast<char>([mOwner message: SCI_GETSTYLEAT wParam: posRange.location]);
 	const long style = static_cast<unsigned char>(styleByte);
-	std::string fontName([mOwner message: SCI_STYLEGETFONT wParam: style lParam: 0] + 1, 0);
+	std::string fontName([mOwner message: SCI_STYLEGETFONT wParam: style lParam: 0], 0);
 	[mOwner message: SCI_STYLEGETFONT wParam: style lParam: (sptr_t)&fontName[0]];
 	const CGFloat fontSize = [mOwner message: SCI_STYLEGETSIZEFRACTIONAL wParam: style] / 100.0f;
 	NSString *sFontName = @(fontName.c_str());
 	NSFont *font = [NSFont fontWithName: sFontName size: fontSize];
-	[asResult addAttribute: NSFontAttributeName value: font range: rangeAS];
+	if (font) {
+		[asResult addAttribute: NSFontAttributeName value: font range: rangeAS];
+	}
 
 	return asResult;
 }
@@ -525,7 +529,7 @@ static NSCursor *cursorFromEnum(Window::Cursor cursor) {
 	else if ([aString isKindOfClass: [NSAttributedString class]])
 		newText = (NSString *) [aString string];
 
-	mOwner.backend->InsertText(newText);
+	mOwner.backend->InsertText(newText, EditModel::CharacterSource::directInput);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -618,11 +622,11 @@ static NSCursor *cursorFromEnum(Window::Cursor cursor) {
 		NSRange posRangeCurrent = mOwner.backend->PositionsFromCharacters(NSMakeRange(replacementRange.location, 0));
 		// Note: Scintilla internally works almost always with bytes instead chars, so we need to take
 		//       this into account when determining selection ranges and such.
-		ptrdiff_t lengthInserted = mOwner.backend->InsertText(newText);
+		ptrdiff_t lengthInserted = mOwner.backend->InsertText(newText, EditModel::CharacterSource::tentativeInput);
 		posRangeCurrent.length = lengthInserted;
 		mMarkedTextRange = mOwner.backend->CharactersFromPositions(posRangeCurrent);
 		// Mark the just inserted text. Keep the marked range for later reset.
-		[mOwner setGeneralProperty: SCI_SETINDICATORCURRENT value: INDIC_IME];
+		[mOwner setGeneralProperty: SCI_SETINDICATORCURRENT value: INDICATOR_IME];
 		[mOwner setGeneralProperty: SCI_INDICATORFILLRANGE
 				 parameter: posRangeCurrent.location
 				     value: posRangeCurrent.length];
@@ -744,14 +748,14 @@ static NSCursor *cursorFromEnum(Window::Cursor cursor) {
 		// Only snap for positions inside the document - allow outside
 		// for overshoot.
 		long lineHeight = mOwner.backend->WndProc(SCI_TEXTHEIGHT, 0, 0);
-		rc.origin.y = roundf(static_cast<XYPOSITION>(rc.origin.y) / lineHeight) * lineHeight;
+		rc.origin.y = std::round(static_cast<XYPOSITION>(rc.origin.y) / lineHeight) * lineHeight;
 	}
 	// Snap to whole points - on retina displays this avoids visual debris
 	// when scrolling horizontally.
 	if ((rc.origin.x > 0) && (NSMaxX(rc) < contentRect.size.width)) {
 		// Only snap for positions inside the document - allow outside
 		// for overshoot.
-		rc.origin.x = roundf(static_cast<XYPOSITION>(rc.origin.x));
+		rc.origin.x = std::round(static_cast<XYPOSITION>(rc.origin.x));
 	}
 	return rc;
 }
@@ -1233,7 +1237,7 @@ static NSCursor *cursorFromEnum(Window::Cursor cursor) {
 #if MAC_OS_X_VERSION_MAX_ALLOWED > MAC_OS_X_VERSION_10_5
 	zoomDelta += event.magnification * 10.0;
 
-	if (fabs(zoomDelta)>=1.0) {
+	if (std::abs(zoomDelta)>=1.0) {
 		long zoomFactor = static_cast<long>([self getGeneralProperty: SCI_GETZOOM] + zoomDelta);
 		[self setGeneralProperty: SCI_SETZOOM parameter: zoomFactor value: 0];
 		zoomDelta = 0.0;
@@ -1372,11 +1376,11 @@ static NSCursor *cursorFromEnum(Window::Cursor cursor) {
  * input composition, depending on language, keyboard etc.
  */
 - (void) updateIndicatorIME {
-	[self setColorProperty: SCI_INDICSETFORE parameter: INDIC_IME fromHTML: @"#FF0000"];
+	[self setColorProperty: SCI_INDICSETFORE parameter: INDICATOR_IME fromHTML: @"#FF0000"];
 	const bool drawInBackground = [self message: SCI_GETPHASESDRAW] != 0;
-	[self setGeneralProperty: SCI_INDICSETUNDER parameter: INDIC_IME value: drawInBackground];
-	[self setGeneralProperty: SCI_INDICSETSTYLE parameter: INDIC_IME value: INDIC_PLAIN];
-	[self setGeneralProperty: SCI_INDICSETALPHA parameter: INDIC_IME value: 100];
+	[self setGeneralProperty: SCI_INDICSETUNDER parameter: INDICATOR_IME value: drawInBackground];
+	[self setGeneralProperty: SCI_INDICSETSTYLE parameter: INDICATOR_IME value: INDIC_PLAIN];
+	[self setGeneralProperty: SCI_INDICSETALPHA parameter: INDICATOR_IME value: 100];
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -1398,7 +1402,7 @@ static NSCursor *cursorFromEnum(Window::Cursor cursor) {
 #if defined(MAC_OS_X_VERSION_10_14)
 		// Let SCIScrollView account for other subviews such as vertical ruler by turning off
 		// automaticallyAdjustsContentInsets.
-		if (floor(NSAppKitVersionNumber) > NSAppKitVersionNumber10_13) {
+		if (std::floor(NSAppKitVersionNumber) > NSAppKitVersionNumber10_13) {
 			scrollView.contentView.automaticallyAdjustsContentInsets = NO;
 			scrollView.contentView.contentInsets = NSEdgeInsetsMake(0., 0., 0., 0.);
 		}
@@ -1961,9 +1965,9 @@ static NSCursor *cursorFromEnum(Window::Cursor cursor) {
 
 - (void) insertText: (id) aString {
 	if ([aString isKindOfClass: [NSString class]])
-		mBackend->InsertText(aString);
+		mBackend->InsertText(aString, EditModel::CharacterSource::directInput);
 	else if ([aString isKindOfClass: [NSAttributedString class]])
-		mBackend->InsertText([aString string]);
+		mBackend->InsertText([aString string], EditModel::CharacterSource::directInput);
 }
 
 //--------------------------------------------------------------------------------------------------
