@@ -7,23 +7,27 @@
 
 #include <cstddef>
 #include <cstdlib>
+#include <cstdint>
 #include <cstring>
-#include <cctype>
 #include <cstdio>
 #include <ctime>
 
 #include <string>
+#include <string_view>
 #include <vector>
 #include <map>
 #include <set>
 #include <memory>
+#include <chrono>
 #include <sstream>
 
 #include <fcntl.h>
 #include <sys/stat.h>
 
 #include "ILexer.h"
-#include "Scintilla.h"
+
+#include "ScintillaTypes.h"
+#include "ScintillaCall.h"
 
 #include "GUI.h"
 #include "ScintillaWindow.h"
@@ -45,7 +49,7 @@
 
 //---------- Save to TeX ----------
 
-static char* getTexRGB(char* texcolor, const char* stylecolor) {
+static char *getTexRGB(char *texcolor, const char *stylecolor) noexcept {
 	//texcolor[rgb]{0,0.5,0}{....}
 	const double rf = IntFromHexByte(stylecolor + 1) / 256.0;
 	const double gf = IntFromHexByte(stylecolor + 3) / 256.0;
@@ -59,20 +63,20 @@ static char* getTexRGB(char* texcolor, const char* stylecolor) {
 }
 
 #define CHARZ ('z' - 'b')
-static char* texStyle(int style) {
+static char *texStyle(int style) noexcept {
 	static char buf[10];
 	int i = 0;
 	do {
 		buf[i++] = static_cast<char>('a' + (style % CHARZ));
 		style /= CHARZ;
-	} while ( style > 0 );
+	} while (style > 0);
 	buf[i] = 0;
 	return buf;
 }
 
-static void defineTexStyle(const StyleDefinition &style, FILE* fp, int istyle) {
+static void defineTexStyle(const StyleDefinition &style, FILE *fp, int istyle) {
 	int closing_brackets = 2;
-	char rgb[200];
+	char rgb[200] = "";
 	fprintf(fp, "\\newcommand{\\scite%s}[1]{\\noindent{\\ttfamily{", texStyle(istyle));
 	if (style.italics) {
 		fputs("\\textit{", fp);
@@ -83,41 +87,37 @@ static void defineTexStyle(const StyleDefinition &style, FILE* fp, int istyle) {
 		closing_brackets++;
 	}
 	if (style.fore.length()) {
-		fprintf(fp, "\\textcolor[rgb]{%s}{", getTexRGB(rgb, style.fore.c_str()) );
+		fprintf(fp, "\\textcolor[rgb]{%s}{", getTexRGB(rgb, style.fore.c_str()));
 		closing_brackets++;
 	}
 	if (style.back.length()) {
-		fprintf(fp, "\\colorbox[rgb]{%s}{", getTexRGB( rgb, style.back.c_str()) );
+		fprintf(fp, "\\colorbox[rgb]{%s}{", getTexRGB(rgb, style.back.c_str()));
 		closing_brackets++;
 	}
 	fputs("#1", fp);
 	for (int i = 0; i <= closing_brackets; i++) {
-		fputc( '}', fp );
+		fputc('}', fp);
 	}
 	fputc('\n', fp);
 }
 
 void SciTEBase::SaveToTEX(const FilePath &saveName) {
 	RemoveFindMarks();
-	wEditor.Call(SCI_COLOURISE, 0, -1);
+	wEditor.ColouriseAll();
 	int tabSize = props.GetInt("tabsize");
 	if (tabSize == 0)
 		tabSize = 4;
 
-	const int lengthDoc = LengthDocument();
+	const SA::Position lengthDoc = LengthDocument();
 	TextReader acc(wEditor);
-	bool styleIsUsed[STYLE_MAX + 1];
+	bool styleIsUsed[StyleMax + 1] = {};
 
 	const int titleFullPath = props.GetInt("export.tex.title.fullpath", 0);
 
-	int i;
-	for (i = 0; i <= STYLE_MAX; i++) {
-		styleIsUsed[i] = false;
+	for (SA::Position pos = 0; pos < lengthDoc; pos++) {	// check the used styles
+		styleIsUsed[acc.StyleAt(pos)] = true;
 	}
-	for (i = 0; i < lengthDoc; i++) {	// check the used styles
-		styleIsUsed[acc.StyleAt(i)] = true;
-	}
-	styleIsUsed[STYLE_DEFAULT] = true;
+	styleIsUsed[StyleDefault] = true;
 
 	FILE *fp = saveName.Open(GUI_TEXT("wt"));
 	bool failedWrite = fp == nullptr;
@@ -127,19 +127,19 @@ void SciTEBase::SaveToTEX(const FilePath &saveName) {
 		      "\\usepackage[T1]{fontenc}\n"
 		      "\\usepackage{color}\n"
 		      "\\usepackage{alltt}\n"
- 		      "\\usepackage{times}\n"
- 		      "\\setlength{\\fboxsep}{0pt}\n", fp);
+		      "\\usepackage{times}\n"
+		      "\\setlength{\\fboxsep}{0pt}\n", fp);
 
-		for (i = 0; i < STYLE_MAX; i++) {      // get keys
-			if (styleIsUsed[i]) {
-				StyleDefinition sd = StyleDefinitionFor(i);
-				defineTexStyle(sd, fp, i); // writeout style macroses
+		for (int istyle = 0; istyle < StyleMax; istyle++) {      // get keys
+			if (styleIsUsed[istyle]) {
+				StyleDefinition sd = StyleDefinitionFor(istyle);
+				defineTexStyle(sd, fp, istyle); // writeout style macroses
 			}
 		}
 
 		fputs("\\begin{document}\n\n", fp);
 		fprintf(fp, "Source File: %s\n\n\\noindent\n\\small{\n",
-		        titleFullPath ? filePath.AsUTF8().c_str() : filePath.Name().AsUTF8().c_str());
+			titleFullPath ? filePath.AsUTF8().c_str() : filePath.Name().AsUTF8().c_str());
 
 		int styleCurrent = acc.StyleAt(0);
 
@@ -147,16 +147,16 @@ void SciTEBase::SaveToTEX(const FilePath &saveName) {
 
 		int lineIdx = 0;
 
-		for (i = 0; i < lengthDoc; i++) { //here process each character of the document
+		for (SA::Position i = 0; i < lengthDoc; i++) { //here process each character of the document
 			const char ch = acc[i];
 			const int style = acc.StyleAt(i);
 
 			if (style != styleCurrent) { //new style?
-				fprintf(fp, "}\\scite%s{", texStyle(style) );
+				fprintf(fp, "}\\scite%s{", texStyle(style));
 				styleCurrent = style;
 			}
 
-			switch ( ch ) { //write out current character.
+			switch (ch) {   //write out current character.
 			case '\t': {
 					const int ts = tabSize - (lineIdx % tabSize);
 					lineIdx += ts - 1;
@@ -188,7 +188,7 @@ void SciTEBase::SaveToTEX(const FilePath &saveName) {
 				if (ch == '\r' && acc[i + 1] == '\n')
 					i++;	// Skip the LF
 				styleCurrent = acc.StyleAt(i + 1);
-				fprintf(fp, "} \\\\\n\\scite%s{", texStyle(styleCurrent) );
+				fprintf(fp, "} \\\\\n\\scite%s{", texStyle(styleCurrent));
 				break;
 			case ' ':
 				if (acc[i + 1] == ' ') {
