@@ -11,21 +11,24 @@
 #define __STDC_LIMIT_MACROS
 #include <cstdint>
 #include <cstring>
-#include <cctype>
 #include <cstdio>
 #include <ctime>
 
 #include <string>
+#include <string_view>
 #include <vector>
 #include <map>
 #include <set>
 #include <algorithm>
 #include <memory>
+#include <chrono>
 
 #include <fcntl.h>
 
 #include "ILoader.h"
-#include "Scintilla.h"
+
+#include "ScintillaTypes.h"
+#include "ScintillaCall.h"
 
 #include "GUI.h"
 #include "ScintillaWindow.h"
@@ -111,11 +114,11 @@ void SciTEBase::CountLineEnds(int &linesCR, int &linesLF, int &linesCRLF) {
 	linesCR = 0;
 	linesLF = 0;
 	linesCRLF = 0;
-	const int lengthDoc = LengthDocument();
+	const SA::Position lengthDoc = LengthDocument();
 	char chPrev = ' ';
 	TextReader acc(wEditor);
 	char chNext = acc.SafeGetCharAt(0);
-	for (int i = 0; i < lengthDoc; i++) {
+	for (SA::Position i = 0; i < lengthDoc; i++) {
 		const char ch = chNext;
 		chNext = acc.SafeGetCharAt(i + 1);
 		if (ch == '\r') {
@@ -142,18 +145,18 @@ void SciTEBase::DiscoverEOLSetting() {
 		int linesCRLF;
 		CountLineEnds(linesCR, linesLF, linesCRLF);
 		if (((linesLF >= linesCR) && (linesLF > linesCRLF)) || ((linesLF > linesCR) && (linesLF >= linesCRLF)))
-			wEditor.Call(SCI_SETEOLMODE, SC_EOL_LF);
+			wEditor.SetEOLMode(SA::EndOfLine::Lf);
 		else if (((linesCR >= linesLF) && (linesCR > linesCRLF)) || ((linesCR > linesLF) && (linesCR >= linesCRLF)))
-			wEditor.Call(SCI_SETEOLMODE, SC_EOL_CR);
+			wEditor.SetEOLMode(SA::EndOfLine::Cr);
 		else if (((linesCRLF >= linesLF) && (linesCRLF > linesCR)) || ((linesCRLF > linesLF) && (linesCRLF >= linesCR)))
-			wEditor.Call(SCI_SETEOLMODE, SC_EOL_CRLF);
+			wEditor.SetEOLMode(SA::EndOfLine::CrLf);
 	}
 }
 
 // Look inside the first line for a #! clue regarding the language
 std::string SciTEBase::DiscoverLanguage() {
-	const int length = Minimum(LengthDocument(), 64 * 1024);
-	std::string buf = GetRangeString(wEditor, 0, length);
+	const SA::Position length = std::min<SA::Position>(LengthDocument(), 64 * 1024);
+	std::string buf = wEditor.StringOfRange(SA::Range(0, length));
 	std::string languageOverride = "";
 	std::string l1 = ExtractLine(buf.c_str(), length);
 	if (StartsWith(l1, "<?xml")) {
@@ -191,7 +194,7 @@ std::string SciTEBase::DiscoverLanguage() {
 }
 
 void SciTEBase::DiscoverIndentSetting() {
-	const int lengthDoc = std::min(LengthDocument(), 1000000);
+	const SA::Position lengthDoc = std::min<SA::Position>(LengthDocument(), 1000000);
 	TextReader acc(wEditor);
 	bool newline = true;
 	int indent = 0; // current line indentation
@@ -236,10 +239,10 @@ void SciTEBase::DiscoverIndentSetting() {
 	}
 	// set indentation
 	if (topTabSize == 0) {
-		wEditor.Call(SCI_SETUSETABS, 1);
+		wEditor.SetUseTabs(true);
 	} else if (topTabSize != -1) {
-		wEditor.Call(SCI_SETUSETABS, 0);
-		wEditor.Call(SCI_SETINDENT, topTabSize);
+		wEditor.SetUseTabs(false);
+		wEditor.SetIndent(topTabSize);
 	}
 }
 
@@ -259,47 +262,48 @@ void SciTEBase::OpenCurrentFile(long long fileSize, bool suppressMessage, bool a
 			GUI::gui_string msg = LocaliseMessage("Could not open file '^0'.", filePath.AsInternal());
 			WindowMessageBox(wSciTE, msg);
 		}
-		if (!wEditor.Call(SCI_GETUNDOCOLLECTION)) {
-			wEditor.Call(SCI_SETUNDOCOLLECTION, 1);
+		if (!wEditor.UndoCollection()) {
+			wEditor.SetUndoCollection(true);
 		}
 		return;
 	}
 
 	CurrentBuffer()->SetTimeFromFile();
 
-	wEditor.Call(SCI_BEGINUNDOACTION);	// Group together clear and insert
-	wEditor.Call(SCI_CLEARALL);
+	wEditor.BeginUndoAction();	// Group together clear and insert
+	wEditor.ClearAll();
 
 	CurrentBuffer()->lifeState = Buffer::reading;
 	if (asynchronous) {
 		// Turn grey while loading
-		wEditor.Call(SCI_STYLESETBACK, STYLE_DEFAULT, 0xEEEEEE);
-		wEditor.Call(SCI_SETREADONLY, 1);
+		wEditor.StyleSetBack(StyleDefault, 0xEEEEEE);
+		wEditor.SetReadOnly(true);
 		assert(CurrentBufferConst()->pFileWorker == nullptr);
 		ILoader *pdocLoad;
 		try {
-			sptr_t docOptions = SC_DOCUMENTOPTION_DEFAULT;
+			SA::DocumentOption docOptions = SA::DocumentOption::Default;
 
 			const long long sizeLarge = props.GetLongLong("file.size.large");
 			if (sizeLarge && (fileSize > sizeLarge))
-				docOptions |= SC_DOCUMENTOPTION_TEXT_LARGE;
+				docOptions = SA::DocumentOption::TextLarge;
 
 			const long long sizeNoStyles = props.GetLongLong("file.size.no.styles");
 			if (sizeNoStyles && (fileSize > sizeNoStyles))
-				docOptions |= SC_DOCUMENTOPTION_STYLES_NONE;
+				docOptions = static_cast<SA::DocumentOption>(
+						     static_cast<int>(docOptions) | static_cast<int>(SA::DocumentOption::StylesNone));
 
 			pdocLoad = reinterpret_cast<ILoader *>(
-				wEditor.CallReturnPointer(SCI_CREATELOADER, static_cast<uptr_t>(fileSize) + 1000,
-					docOptions));
+					   wEditor.CreateLoader(static_cast<SA::Position>(fileSize) + 1000,
+								docOptions));
 		} catch (...) {
-			wEditor.Call(SCI_SETSTATUS, 0);
+			wEditor.SetStatus(SA::Status::Ok);
 			return;
 		}
 		CurrentBuffer()->pFileWorker = new FileLoader(this, pdocLoad, filePath, static_cast<size_t>(fileSize), fp);
 		CurrentBuffer()->pFileWorker->sleepTime = props.GetInt("asynchronous.sleep");
 		PerformOnNewThread(CurrentBuffer()->pFileWorker);
 	} else {
-		wEditor.Call(SCI_ALLOCATE, static_cast<uptr_t>(fileSize) + 1000);
+		wEditor.Allocate(static_cast<SA::Position>(fileSize) + 1000);
 
 		Utf8_16_Read convert;
 		std::vector<char> data(blockSize);
@@ -308,22 +312,22 @@ void SciTEBase::OpenCurrentFile(long long fileSize, bool suppressMessage, bool a
 		while (lenFile > 0) {
 			lenFile = convert.convert(&data[0], lenFile);
 			const char *dataBlock = convert.getNewBuf();
-			wEditor.CallString(SCI_ADDTEXT, lenFile, dataBlock);
+			wEditor.AddText(lenFile, dataBlock);
 			lenFile = fread(&data[0], 1, data.size(), fp);
 			if (lenFile == 0) {
 				// Handle case where convert is holding a lead surrogate but no more data
 				const size_t lenFileTrail = convert.convert(nullptr, lenFile);
 				if (lenFileTrail) {
 					const char *dataTrail = convert.getNewBuf();
-					wEditor.CallString(SCI_ADDTEXT, lenFileTrail, dataTrail);
+					wEditor.AddText(lenFileTrail, dataTrail);
 				}
 			}
 		}
 		fclose(fp);
-		wEditor.Call(SCI_ENDUNDOACTION);
+		wEditor.EndUndoAction();
 
 		CurrentBuffer()->unicodeMode = static_cast<UniMode>(
-			    static_cast<int>(convert.getEncoding()));
+						       static_cast<int>(convert.getEncoding()));
 		// Check the first two lines for coding cookies
 		if (CurrentBuffer()->unicodeMode == uni8Bit) {
 			CurrentBuffer()->unicodeMode = umCodingCookie;
@@ -347,7 +351,7 @@ void SciTEBase::TextRead(FileWorker *pFileWorker) {
 			buffers.buffers[iBuffer].lifeState = Buffer::empty;
 		}
 		// Switch documents
-		const sptr_t pdocLoading = reinterpret_cast<sptr_t>(pFileLoader->pLoader->ConvertToDocument());
+		void *pdocLoading = pFileLoader->pLoader->ConvertToDocument();
 		pFileLoader->pLoader = nullptr;
 		SwitchDocumentAt(iBuffer, pdocLoading);
 		if (iBuffer == buffers.Current()) {
@@ -356,21 +360,21 @@ void SciTEBase::TextRead(FileWorker *pFileWorker) {
 				extender->OnOpen(buffers.buffers[iBuffer].file.AsUTF8().c_str());
 			RestoreState(buffers.buffers[iBuffer], true);
 			DisplayAround(buffers.buffers[iBuffer].file);
-			wEditor.Call(SCI_SCROLLCARET);
+			wEditor.ScrollCaret();
 		}
 	}
 }
 
 void SciTEBase::PerformDeferredTasks() {
 	if (buffers.buffers[buffers.Current()].futureDo & Buffer::fdFinishSave) {
-		wEditor.Call(SCI_SETSAVEPOINT);
-		wEditor.Call(SCI_SETREADONLY, CurrentBuffer()->isReadOnly);
+		wEditor.SetSavePoint();
+		wEditor.SetReadOnly(CurrentBuffer()->isReadOnly);
 		buffers.FinishedFuture(buffers.Current(), Buffer::fdFinishSave);
 	}
 }
 
 void SciTEBase::CompleteOpen(OpenCompletion oc) {
-	wEditor.Call(SCI_SETREADONLY, CurrentBuffer()->isReadOnly);
+	wEditor.SetReadOnly(CurrentBuffer()->isReadOnly);
 
 	if (oc != ocSynchronous) {
 		ReadProperties();
@@ -395,11 +399,11 @@ void SciTEBase::CompleteOpen(OpenCompletion oc) {
 
 	if (CurrentBuffer()->unicodeMode != uni8Bit) {
 		// Override the code page if Unicode
-		codePage = SC_CP_UTF8;
+		codePage = SA::CpUtf8;
 	} else {
 		codePage = props.GetInt("code.page");
 	}
-	wEditor.Call(SCI_SETCODEPAGE, codePage);
+	wEditor.SetCodePage(codePage);
 
 	DiscoverEOLSetting();
 
@@ -407,14 +411,14 @@ void SciTEBase::CompleteOpen(OpenCompletion oc) {
 		DiscoverIndentSetting();
 	}
 
-	if (!wEditor.Call(SCI_GETUNDOCOLLECTION)) {
-		wEditor.Call(SCI_SETUNDOCOLLECTION, 1);
+	if (!wEditor.UndoCollection()) {
+		wEditor.SetUndoCollection(true);
 	}
-	wEditor.Call(SCI_SETSAVEPOINT);
+	wEditor.SetSavePoint();
 	if (props.GetInt("fold.on.open") > 0) {
 		FoldAll();
 	}
-	wEditor.Call(SCI_GOTOPOS, 0);
+	wEditor.GotoPos(0);
 
 	CurrentBuffer()->CompleteLoading();
 
@@ -439,16 +443,16 @@ void SciTEBase::TextWritten(FileWorker *pFileWorker) {
 			buffers.SetVisible(iBuffer, true);
 			SetBuffersMenu();
 			if (iBuffer == buffers.Current()) {
-				wEditor.Call(SCI_SETREADONLY, CurrentBuffer()->isReadOnly);
+				wEditor.SetReadOnly(CurrentBuffer()->isReadOnly);
 			}
 		} else {
 			if (!buffers.GetVisible(iBuffer)) {
 				buffers.RemoveInvisible(iBuffer);
 			}
 			if (iBuffer == buffers.Current()) {
-				wEditor.Call(SCI_SETREADONLY, CurrentBuffer()->isReadOnly);
+				wEditor.SetReadOnly(CurrentBuffer()->isReadOnly);
 				if (pathSaved.SameNameAs(CurrentBuffer()->file)) {
-					wEditor.Call(SCI_SETSAVEPOINT);
+					wEditor.SetSavePoint();
 				}
 				if (extender)
 					extender->OnSave(buffers.buffers[iBuffer].file.AsUTF8().c_str());
@@ -491,7 +495,7 @@ void SciTEBase::UpdateProgress(Worker *) {
 	} else {
 		if (countBoth == 1) {
 			prog += LocaliseMessage(bgActivities.loaders ? "Opening '^0'" : "Saving '^0'",
-				bgActivities.fileNameLast.c_str());
+						bgActivities.fileNameLast.c_str());
 		} else {
 			if (bgActivities.loaders) {
 				prog += LocaliseMessage("Opening ^0 files ", GUI::StringFromInteger(bgActivities.loaders).c_str());
@@ -514,7 +518,7 @@ bool SciTEBase::Open(const FilePath &file, OpenFlags of) {
 	FilePath absPath = file.AbsolutePath();
 	if (!absPath.IsUntitled() && absPath.IsDirectory()) {
 		GUI::gui_string msg = LocaliseMessage("Path '^0' is a directory so can not be opened.",
-			absPath.AsInternal());
+						      absPath.AsInternal());
 		WindowMessageBox(wSciTE, msg);
 		return false;
 	}
@@ -539,8 +543,8 @@ bool SciTEBase::Open(const FilePath &file, OpenFlags of) {
 	if (fileSize > INTPTR_MAX) {
 		const GUI::gui_string sSize = GUI::StringFromLongLong(fileSize);
 		const GUI::gui_string msg = LocaliseMessage("File '^0' is ^1 bytes long, "
-			"larger than 2GB which is the largest SciTE can open.",
-			absPath.AsInternal(), sSize.c_str());
+					    "larger than 2GB which is the largest SciTE can open.",
+					    absPath.AsInternal(), sSize.c_str());
 		WindowMessageBox(wSciTE, msg, mbsIconWarning);
 		return false;
 	}
@@ -551,9 +555,9 @@ bool SciTEBase::Open(const FilePath &file, OpenFlags of) {
 			const GUI::gui_string sSize = GUI::StringFromLongLong(fileSize);
 			const GUI::gui_string sMaxSize = GUI::StringFromLongLong(maxSize);
 			const GUI::gui_string msg = LocaliseMessage("File '^0' is ^1 bytes long,\n"
-			        "larger than the ^2 bytes limit set in the properties.\n"
-			        "Do you still want to open it?",
-			        absPath.AsInternal(), sSize.c_str(), sMaxSize.c_str());
+						    "larger than the ^2 bytes limit set in the properties.\n"
+						    "Do you still want to open it?",
+						    absPath.AsInternal(), sSize.c_str(), sMaxSize.c_str());
 			const MessageBoxChoice answer = WindowMessageBox(wSciTE, msg, mbsYesNo | mbsIconWarning);
 			if (answer != mbYes) {
 				return false;
@@ -580,7 +584,7 @@ bool SciTEBase::Open(const FilePath &file, OpenFlags of) {
 	std::string discoveryScript = props.GetExpandedString("command.discover.properties");
 	if (discoveryScript.length()) {
 		std::string propertiesText = CommandExecute(GUI::StringFromUTF8(discoveryScript).c_str(),
-			absPath.Directory().AsInternal());
+					     absPath.Directory().AsInternal());
 		if (propertiesText.size()) {
 			propsDiscovered.ReadFromMemory(propertiesText.c_str(), propertiesText.size(), absPath.Directory(), filter, nullptr, 0);
 		}
@@ -596,25 +600,25 @@ bool SciTEBase::Open(const FilePath &file, OpenFlags of) {
 
 	bool asynchronous = false;
 	if (!filePath.IsUntitled()) {
-		wEditor.Call(SCI_SETREADONLY, 0);
-		wEditor.Call(SCI_CANCEL);
+		wEditor.SetReadOnly(false);
+		wEditor.Cancel();
 		if (of & ofPreserveUndo) {
-			wEditor.Call(SCI_BEGINUNDOACTION);
+			wEditor.BeginUndoAction();
 		} else {
-			wEditor.Call(SCI_SETUNDOCOLLECTION, 0);
+			wEditor.SetUndoCollection(false);
 		}
 
 		asynchronous = (fileSize > props.GetInt("background.open.size", -1)) &&
-			!(of & (ofPreserveUndo|ofSynchronous));
+			       !(of & (ofPreserveUndo|ofSynchronous));
 		OpenCurrentFile(fileSize, of & ofQuiet, asynchronous);
 
 		if (of & ofPreserveUndo) {
-			wEditor.Call(SCI_ENDUNDOACTION);
+			wEditor.EndUndoAction();
 		} else {
-			wEditor.Call(SCI_EMPTYUNDOBUFFER);
+			wEditor.EmptyUndoBuffer();
 		}
 		CurrentBuffer()->isReadOnly = props.GetInt("read.only");
-		wEditor.Call(SCI_SETREADONLY, CurrentBuffer()->isReadOnly);
+		wEditor.SetReadOnly(CurrentBuffer()->isReadOnly);
 	}
 	RemoveFileFromStack(filePath);
 	DeleteFileStackMenu();
@@ -638,11 +642,11 @@ bool SciTEBase::OpenSelected() {
 
 #if !defined(GTK)
 	if (StartsWith(selName, "http:") ||
-		StartsWith(selName, "https:") ||
-		StartsWith(selName, "ftp:") ||
-		StartsWith(selName, "ftps:") ||
-		StartsWith(selName, "news:") ||
-		StartsWith(selName, "mailto:")) {
+			StartsWith(selName, "https:") ||
+			StartsWith(selName, "ftp:") ||
+			StartsWith(selName, "ftps:") ||
+			StartsWith(selName, "news:") ||
+			StartsWith(selName, "mailto:")) {
 		std::string cmd = selName;
 		AddCommand(cmd, "", jobShell);
 		return false;	// Job is done
@@ -666,14 +670,14 @@ bool SciTEBase::OpenSelected() {
 	}
 
 	std::string cTag;
-	unsigned long lineNumber = 0;
+	SA::Line lineNumber = 0;
 	if (IsPropertiesFile(filePath) &&
-	        (selName.find('.') == std::string::npos)) {
+			(selName.find('.') == std::string::npos)) {
 		// We are in a properties file and try to open a file without extension,
 		// we suppose we want to open an imported .properties file
 		// So we append the correct extension to open the included file.
 		// Maybe we should check if the filename is preceded by "import"...
-		selName += PROPERTIES_EXTENSION;
+		selName += extensionProperties;
 	} else {
 		// Check if we have a line number (error message or grep result)
 		// A bit of duplicate work with DecodeMessage, but we don't know
@@ -708,7 +712,7 @@ bool SciTEBase::OpenSelected() {
 		// If not there, look in openpath
 		if (!Exists(path.AsInternal(), selFN.c_str(), nullptr)) {
 			GUI::gui_string openPath = GUI::StringFromUTF8(props.GetNewExpandString(
-			            "openpath.", fileNameForExtension.c_str()));
+							   "openpath.", fileNameForExtension.c_str()));
 			while (openPath.length()) {
 				GUI::gui_string tryPath(openPath);
 				const size_t sepIndex = tryPath.find(listSepString);
@@ -731,10 +735,11 @@ bool SciTEBase::OpenSelected() {
 		const OpenFlags of = ((lineNumber > 0) || (cTag.length() != 0)) ? ofSynchronous : ofNone;
 		if (Open(pathReturned, of)) {
 			if (lineNumber > 0) {
-				wEditor.Call(SCI_GOTOLINE, lineNumber - 1);
+				wEditor.GotoLine(lineNumber - 1);
 			} else if (cTag.length() != 0) {
-				if (atoi(cTag.c_str()) > 0) {
-					wEditor.Call(SCI_GOTOLINE, atoi(cTag.c_str()) - 1);
+				const SA::Line cTagLine = IntegerFromText(cTag.c_str());
+				if (cTagLine > 0) {
+					wEditor.GotoLine(cTagLine - 1);
 				} else {
 					findWhat = cTag;
 					FindNext(false);
@@ -750,7 +755,7 @@ bool SciTEBase::OpenSelected() {
 
 void SciTEBase::Revert() {
 	if (filePath.IsUntitled()) {
-		wEditor.Call(SCI_CLEARALL);
+		wEditor.ClearAll();
 	} else {
 		RecentFile rf = GetFilePosition();
 		OpenCurrentFile(filePath.GetFileLength(), false, false);
@@ -770,12 +775,12 @@ void SciTEBase::CheckReload() {
 					GUI::gui_string msg;
 					if (CurrentBuffer()->isDirty) {
 						msg = LocaliseMessage(
-						          "The file '^0' has been modified. Should it be reloaded?",
-						          filePath.AsInternal());
+							      "The file '^0' has been modified. Should it be reloaded?",
+							      filePath.AsInternal());
 					} else {
 						msg = LocaliseMessage(
-						          "The file '^0' has been modified outside SciTE. Should it be reloaded?",
-						          FileNameExt().AsInternal());
+							      "The file '^0' has been modified outside SciTE. Should it be reloaded?",
+							      FileNameExt().AsInternal());
 					}
 					const MessageBoxChoice decision = WindowMessageBox(wSciTE, msg, mbsYesNo | mbsIconQuestion);
 					if (decision == mbYes) {
@@ -846,9 +851,9 @@ SciTEBase::SaveResult SciTEBase::SaveIfUnsure(bool forceQuestion, SaveFlags sf)
 	}
 	if ((CurrentBuffer()->isDirty) && (LengthDocument() || !filePath.IsUntitled() || forceQuestion)) {
 		if (props.GetInt("are.you.sure", 1) ||
-		        filePath.IsUntitled() ||
-		        forceQuestion) {
-					GUI::gui_string msg;
+				filePath.IsUntitled() ||
+				forceQuestion) {
+			GUI::gui_string msg;
 			if (!filePath.IsUntitled()) {
 				msg = LocaliseMessage("Save changes to '^0'?", filePath.AsInternal());
 			} else {
@@ -893,13 +898,13 @@ SciTEBase::SaveResult SciTEBase::SaveIfUnsureAll() {
 	// Definitely going to exit now, so delete all documents
 	// Set editor back to initial document
 	if (buffers.lengthVisible > 0) {
-		wEditor.Call(SCI_SETDOCPOINTER, 0, buffers.buffers[0].doc);
+		wEditor.SetDocPointer(buffers.buffers[0].doc);
 	}
 	// Release all the extra documents
 	for (int j = 0; j < buffers.size(); j++) {
 		if (buffers.buffers[j].doc && !buffers.buffers[j].pFileWorker) {
-			wEditor.Call(SCI_RELEASEDOCUMENT, 0, buffers.buffers[j].doc);
-			buffers.buffers[j].doc = 0;
+			wEditor.ReleaseDocument(buffers.buffers[j].doc);
+			buffers.buffers[j].doc = nullptr;
 		}
 	}
 	// Initial document will be deleted when editor deleted
@@ -929,9 +934,11 @@ SciTEBase::SaveResult SciTEBase::SaveIfUnsureForBuilt() {
 class SelectionKeeper {
 public:
 	explicit SelectionKeeper(GUI::ScintillaWindow &editor) : wEditor(editor) {
-		const int mask = SCVS_RECTANGULARSELECTION | SCVS_USERACCESSIBLE;
-		if (wEditor.Call(SCI_GETVIRTUALSPACEOPTIONS, 0, 0) & mask) {
-			const int n = wEditor.Call(SCI_GETSELECTIONS, 0, 0);
+		const SA::VirtualSpace mask = static_cast<SA::VirtualSpace>(
+						      static_cast<int>(SA::VirtualSpace::RectangularSelection) |
+						      static_cast<int>(SA::VirtualSpace::UserAccessible));
+		if (static_cast<int>(wEditor.VirtualSpaceOptions()) & static_cast<int>(mask)) {
+			const int n = wEditor.Selections();
 			for (int i = 0; i < n; ++i) {
 				selections.push_back(LocFromPos(GetSelection(i)));
 			}
@@ -948,26 +955,26 @@ public:
 
 private:
 	struct Position {
-		Position(int pos_, int virt_ = 0) : pos(pos_), virt(virt_) {};
-		int pos;
-		int virt;
+		Position(SA::Position pos_, SA::Position virt_ = 0) : pos(pos_), virt(virt_) {};
+		SA::Position pos;
+		SA::Position virt;
 	};
 
 	struct Location {
-		Location(int line_, int col_) : line(line_), col(col_) {};
-		int line;
-		int col;
+		Location(SA::Line line_, SA::Position col_) : line(line_), col(col_) {};
+		SA::Line line;
+		SA::Position col;
 	};
 
 	Position GetAnchor(int i) {
-		const int pos  = wEditor.Call(SCI_GETSELECTIONNANCHOR, i, 0);
-		const int virt = wEditor.Call(SCI_GETSELECTIONNANCHORVIRTUALSPACE, i, 0);
+		const SA::Position pos  = wEditor.SelectionNAnchor(i);
+		const SA::Position virt = wEditor.SelectionNAnchorVirtualSpace(i);
 		return Position(pos, virt);
 	}
 
 	Position GetCaret(int i) {
-		const int pos  = wEditor.Call(SCI_GETSELECTIONNCARET, i, 0);
-		const int virt = wEditor.Call(SCI_GETSELECTIONNCARETVIRTUALSPACE, i, 0);
+		const SA::Position pos  = wEditor.SelectionNCaret(i);
+		const SA::Position virt = wEditor.SelectionNCaretVirtualSpace(i);
 		return Position(pos, virt);
 	}
 
@@ -976,8 +983,8 @@ private:
 	};
 
 	Location LocFromPos(Position const &pos) {
-		const int line = wEditor.Call(SCI_LINEFROMPOSITION, pos.pos, 0);
-		const int col  = wEditor.Call(SCI_GETCOLUMN, pos.pos, 0) + pos.virt;
+		const SA::Line line = wEditor.LineFromPosition(pos.pos);
+		const SA::Position col  = wEditor.Column(pos.pos) + pos.virt;
 		return Location(line, col);
 	}
 
@@ -986,8 +993,8 @@ private:
 	}
 
 	Position PosFromLoc(Location const &loc) {
-		const int pos = wEditor.Call(SCI_FINDCOLUMN, loc.line, loc.col);
-		const int col = wEditor.Call(SCI_GETCOLUMN, pos, 0);
+		const SA::Position pos = wEditor.FindColumn(loc.line, loc.col);
+		const SA::Position col = wEditor.Column(pos);
 		return Position(pos, loc.col - col);
 	}
 
@@ -996,13 +1003,13 @@ private:
 	}
 
 	void SetAnchor(int i, Position const &pos) {
-		wEditor.Call(SCI_SETSELECTIONNANCHOR, i, pos.pos);
-		wEditor.Call(SCI_SETSELECTIONNANCHORVIRTUALSPACE, i, pos.virt);
+		wEditor.SetSelectionNAnchor(i, pos.pos);
+		wEditor.SetSelectionNAnchorVirtualSpace(i, pos.virt);
 	};
 
 	void SetCaret(int i, Position const &pos) {
-		wEditor.Call(SCI_SETSELECTIONNCARET, i, pos.pos);
-		wEditor.Call(SCI_SETSELECTIONNCARETVIRTUALSPACE, i, pos.virt);
+		wEditor.SetSelectionNCaret(i, pos.pos);
+		wEditor.SetSelectionNCaretVirtualSpace(i, pos.virt);
 	}
 
 	void SetSelection(int i, std::pair<Position, Position> const &pos) {
@@ -1015,43 +1022,44 @@ private:
 };
 
 void SciTEBase::StripTrailingSpaces() {
-	const int maxLines = wEditor.Call(SCI_GETLINECOUNT);
+	const SA::Line maxLines = wEditor.LineCount();
 	SelectionKeeper keeper(wEditor);
 	for (int line = 0; line < maxLines; line++) {
-		const int lineStart = wEditor.Call(SCI_POSITIONFROMLINE, line);
-		const int lineEnd = wEditor.Call(SCI_GETLINEENDPOSITION, line);
-		int i = lineEnd - 1;
-		char ch = static_cast<char>(wEditor.Call(SCI_GETCHARAT, i));
+		const SA::Position lineStart = wEditor.LineStart(line);
+		const SA::Position lineEnd = wEditor.LineEnd(line);
+		SA::Position i = lineEnd - 1;
+		char ch = wEditor.CharacterAt(i);
 		while ((i >= lineStart) && ((ch == ' ') || (ch == '\t'))) {
 			i--;
-			ch = static_cast<char>(wEditor.Call(SCI_GETCHARAT, i));
+			ch = wEditor.CharacterAt(i);
 		}
 		if (i < (lineEnd - 1)) {
-			wEditor.Call(SCI_SETTARGETSTART, i + 1);
-			wEditor.Call(SCI_SETTARGETEND, lineEnd);
-			wEditor.CallString(SCI_REPLACETARGET, 0, "");
+			wEditor.SetTarget(SA::Range(i + 1, lineEnd));
+			wEditor.ReplaceTarget("");
 		}
 	}
 }
 
 void SciTEBase::EnsureFinalNewLine() {
-	const int maxLines = wEditor.Call(SCI_GETLINECOUNT);
+	const SA::Line maxLines = wEditor.LineCount();
 	bool appendNewLine = maxLines == 1;
-	const int endDocument = wEditor.Call(SCI_POSITIONFROMLINE, maxLines);
+	const SA::Position endDocument = wEditor.LineStart(maxLines);
 	if (maxLines > 1) {
-		appendNewLine = endDocument > wEditor.Call(SCI_POSITIONFROMLINE, maxLines - 1);
+		appendNewLine = endDocument > wEditor.LineStart(maxLines - 1);
 	}
 	if (appendNewLine) {
 		const char *eol = "\n";
-		switch (wEditor.Call(SCI_GETEOLMODE)) {
-		case SC_EOL_CRLF:
+		switch (wEditor.EOLMode()) {
+		case SA::EndOfLine::CrLf:
 			eol = "\r\n";
 			break;
-		case SC_EOL_CR:
+		case SA::EndOfLine::Cr:
 			eol = "\r";
 			break;
+		case SA::EndOfLine::Lf:
+			break;
 		}
-		wEditor.CallString(SCI_INSERTTEXT, endDocument, eol);
+		wEditor.InsertText(endDocument, eol);
 	}
 }
 
@@ -1059,18 +1067,18 @@ void SciTEBase::EnsureFinalNewLine() {
 bool SciTEBase::PrepareBufferForSave(const FilePath &saveName) {
 	bool retVal = false;
 	// Perform clean ups on text before saving
-	wEditor.Call(SCI_BEGINUNDOACTION);
+	wEditor.BeginUndoAction();
 	if (stripTrailingSpaces)
 		StripTrailingSpaces();
 	if (ensureFinalLineEnd)
 		EnsureFinalNewLine();
 	if (ensureConsistentLineEnds)
-		wEditor.Call(SCI_CONVERTEOLS, wEditor.Call(SCI_GETEOLMODE));
+		wEditor.ConvertEOLs(wEditor.EOLMode());
 
 	if (extender)
 		retVal = extender->OnBeforeSave(saveName.AsUTF8().c_str());
 
-	wEditor.Call(SCI_ENDUNDOACTION);
+	wEditor.EndUndoAction();
 
 	return retVal;
 }
@@ -1087,8 +1095,8 @@ bool SciTEBase::SaveBuffer(const FilePath &saveName, SaveFlags sf) {
 		if (fp) {
 			const size_t lengthDoc = LengthDocument();
 			if (!(sf & sfSynchronous)) {
-				wEditor.Call(SCI_SETREADONLY, 1);
-				const char *documentBytes = reinterpret_cast<const char *>(wEditor.CallReturnPointer(SCI_GETCHARACTERPOINTER));
+				wEditor.SetReadOnly(true);
+				const char *documentBytes = reinterpret_cast<const char *>(wEditor.CharacterPointer());
 				CurrentBuffer()->pFileWorker = new FileStorer(this, documentBytes, saveName, lengthDoc, fp, CurrentBuffer()->unicodeMode, (sf & sfProgressVisible));
 				CurrentBuffer()->pFileWorker->sleepTime = props.GetInt("asynchronous.sleep");
 				if (PerformOnNewThread(CurrentBuffer()->pFileWorker)) {
@@ -1101,7 +1109,7 @@ bool SciTEBase::SaveBuffer(const FilePath &saveName, SaveFlags sf) {
 				Utf8_16_Write convert;
 				if (CurrentBuffer()->unicodeMode != uniCookie) {	// Save file with cookie without BOM.
 					convert.setEncoding(static_cast<Utf8_16::encodingType>(
-							static_cast<int>(CurrentBuffer()->unicodeMode)));
+								    static_cast<int>(CurrentBuffer()->unicodeMode)));
 				}
 				convert.setfile(fp);
 				std::vector<char> data(blockSize + 1);
@@ -1112,8 +1120,11 @@ bool SciTEBase::SaveBuffer(const FilePath &saveName, SaveFlags sf) {
 					if (grabSize > blockSize)
 						grabSize = blockSize;
 					// Round down so only whole characters retrieved.
-					grabSize = wEditor.Call(SCI_POSITIONBEFORE, i + grabSize + 1) - i;
-					GetRange(wEditor, static_cast<int>(i), static_cast<int>(i + grabSize), &data[0]);
+					grabSize = wEditor.PositionBefore(i + grabSize + 1) - i;
+					const SA::Range rangeGrab(static_cast<SA::Position>(i),
+								  static_cast<SA::Position>(i + grabSize));
+					wEditor.SetTarget(rangeGrab);
+					wEditor.TargetText(&data[0]);
 					const size_t written = convert.fwrite(&data[0], grabSize);
 					if (written == 0) {
 						retVal = false;
@@ -1151,8 +1162,8 @@ bool SciTEBase::Save(SaveFlags sf) {
 		GUI::gui_string msg;
 		if (CurrentBuffer()->ShouldNotSave()) {
 			msg = LocaliseMessage(
-				"The file '^0' has not yet been loaded entirely, so it can not be saved right now. Please retry in a while.",
-				filePath.AsInternal());
+				      "The file '^0' has not yet been loaded entirely, so it can not be saved right now. Please retry in a while.",
+				      filePath.AsInternal());
 			WindowMessageBox(wSciTE, msg);
 			// It is OK to not save this file
 			return true;
@@ -1160,8 +1171,8 @@ bool SciTEBase::Save(SaveFlags sf) {
 
 		if (CurrentBuffer()->pFileWorker) {
 			msg = LocaliseMessage(
-				"The file '^0' is already being saved.",
-				filePath.AsInternal());
+				      "The file '^0' is already being saved.",
+				      filePath.AsInternal());
 			WindowMessageBox(wSciTE, msg);
 			// It is OK to not save this file
 			return true;
@@ -1172,9 +1183,9 @@ bool SciTEBase::Save(SaveFlags sf) {
 		} else if (props.GetInt("save.check.modified.time")) {
 			const time_t newModTime = filePath.ModifiedTime();
 			if ((newModTime != 0) && (CurrentBuffer()->fileModTime != 0) &&
-				(newModTime != CurrentBuffer()->fileModTime)) {
+					(newModTime != CurrentBuffer()->fileModTime)) {
 				msg = LocaliseMessage("The file '^0' has been modified outside SciTE. Should it be saved?",
-					filePath.AsInternal());
+						      filePath.AsInternal());
 				const MessageBoxChoice decision = WindowMessageBox(wSciTE, msg, mbsYesNo | mbsIconQuestion);
 				if (decision == mbNo) {
 					return false;
@@ -1183,12 +1194,12 @@ bool SciTEBase::Save(SaveFlags sf) {
 		}
 
 		if ((LengthDocument() <= props.GetInt("background.save.size", -1)) ||
-			(buffers.SingleBuffer()))
+				(buffers.SingleBuffer()))
 			sf = static_cast<SaveFlags>(sf | sfSynchronous);
 		if (SaveBuffer(filePath, sf)) {
 			CurrentBuffer()->SetTimeFromFile();
 			if (sf & sfSynchronous) {
-				wEditor.Call(SCI_SETSAVEPOINT);
+				wEditor.SetSavePoint();
 				if (IsPropertiesFile(filePath)) {
 					ReloadProperties();
 				}
@@ -1197,7 +1208,7 @@ bool SciTEBase::Save(SaveFlags sf) {
 			if (!CurrentBuffer()->failedSave) {
 				CurrentBuffer()->failedSave = true;
 				msg = LocaliseMessage(
-					"Could not save file '^0'. Save under a different name?", filePath.AsInternal());
+					      "Could not save file '^0'. Save under a different name?", filePath.AsInternal());
 				const MessageBoxChoice decision = WindowMessageBox(wSciTE, msg, mbsYesNo | mbsIconWarning);
 				if (decision == mbYes) {
 					return SaveAsDialog();
@@ -1232,8 +1243,8 @@ void SciTEBase::SaveAs(const GUI::gui_char *file, bool fixCase) {
 	SetFileName(file, fixCase);
 	Save();
 	ReadProperties();
-	wEditor.Call(SCI_CLEARDOCUMENTSTYLE);
-	wEditor.Call(SCI_COLOURISE, 0, wEditor.Call(SCI_POSITIONFROMLINE, 1));
+	wEditor.ClearDocumentStyle();
+	wEditor.Colourise(0, wEditor.LineStart(1));
 	Redraw();
 	SetWindowName();
 	BuffersMenu();
@@ -1246,7 +1257,7 @@ bool SciTEBase::SaveIfNotOpen(const FilePath &destFile, bool fixCase) {
 	const int index = buffers.GetDocumentByName(absPath, true /* excludeCurrent */);
 	if (index >= 0) {
 		GUI::gui_string msg = LocaliseMessage(
-			    "File '^0' is already open in another buffer.", destFile.AsInternal());
+					      "File '^0' is already open in another buffer.", destFile.AsInternal());
 		WindowMessageBox(wSciTE, msg);
 		return false;
 	} else {
@@ -1271,21 +1282,21 @@ void SciTEBase::OpenFromStdin(bool UseOutputPane) {
 	if (IsStdinBlocked())
 		return;
 
-	Open(GUI_TEXT(""));
+	Open(FilePath());
 	if (UseOutputPane) {
-		wOutput.Call(SCI_CLEARALL);
+		wOutput.ClearAll();
 	} else {
-		wEditor.Call(SCI_BEGINUNDOACTION);	// Group together clear and insert
-		wEditor.Call(SCI_CLEARALL);
+		wEditor.BeginUndoAction();	// Group together clear and insert
+		wEditor.ClearAll();
 	}
 	size_t lenFile = fread(&data[0], 1, data.size(), stdin);
 	const UniMode umCodingCookie = CodingCookieValue(&data[0], lenFile);
 	while (lenFile > 0) {
 		lenFile = convert.convert(&data[0], lenFile);
 		if (UseOutputPane) {
-			wOutput.CallString(SCI_ADDTEXT, lenFile, convert.getNewBuf());
+			wOutput.AddText(lenFile, convert.getNewBuf());
 		} else {
-			wEditor.CallString(SCI_ADDTEXT, lenFile, convert.getNewBuf());
+			wEditor.AddText(lenFile, convert.getNewBuf());
 		}
 		lenFile = fread(&data[0], 1, data.size(), stdin);
 	}
@@ -1297,35 +1308,35 @@ void SciTEBase::OpenFromStdin(bool UseOutputPane) {
 		}
 		SizeSubWindows();
 	} else {
-		wEditor.Call(SCI_ENDUNDOACTION);
+		wEditor.EndUndoAction();
 	}
 	CurrentBuffer()->unicodeMode = static_cast<UniMode>(
-	            static_cast<int>(convert.getEncoding()));
+					       static_cast<int>(convert.getEncoding()));
 	// Check the first two lines for coding cookies
 	if (CurrentBuffer()->unicodeMode == uni8Bit) {
 		CurrentBuffer()->unicodeMode = umCodingCookie;
 	}
 	if (CurrentBuffer()->unicodeMode != uni8Bit) {
 		// Override the code page if Unicode
-		codePage = SC_CP_UTF8;
+		codePage = SA::CpUtf8;
 	} else {
 		codePage = props.GetInt("code.page");
 	}
 	if (UseOutputPane) {
-		wOutput.Call(SCI_SETSEL, 0, 0);
+		wOutput.SetSel(0, 0);
 	} else {
-		wEditor.Call(SCI_SETCODEPAGE, codePage);
+		wEditor.SetCodePage(codePage);
 
 		// Zero all the style bytes
-		wEditor.Call(SCI_CLEARDOCUMENTSTYLE);
+		wEditor.ClearDocumentStyle();
 
 		CurrentBuffer()->overrideExtension = "x.txt";
 		ReadProperties();
 		SetIndentSettings();
-		wEditor.Call(SCI_COLOURISE, 0, -1);
+		wEditor.ColouriseAll();
 		Redraw();
 
-		wEditor.Call(SCI_SETSEL, 0, 0);
+		wEditor.SetSel(0, 0);
 	}
 }
 
@@ -1340,10 +1351,10 @@ void SciTEBase::OpenFilesFromStdin() {
 		char *pNL;
 		if ((pNL = strchr(data, '\n')) != nullptr)
 			* pNL = '\0';
-		Open(GUI::StringFromUTF8(data).c_str(), ofQuiet);
+		Open(GUI::StringFromUTF8(data), ofQuiet);
 	}
 	if (buffers.lengthVisible == 0)
-		Open(GUI_TEXT(""));
+		Open(FilePath());
 }
 
 class BufferedFile {
@@ -1354,7 +1365,7 @@ class BufferedFile {
 	char buffer[bufLen];
 	size_t pos;
 	size_t valid;
-	void EnsureData() {
+	void EnsureData() noexcept {
 		if (pos >= valid) {
 			if (readAll || !fp) {
 				exhausted = true;
@@ -1385,16 +1396,16 @@ public:
 	bool Exhausted() const noexcept {
 		return exhausted;
 	}
-	int NextByte() {
+	int NextByte() noexcept {
 		EnsureData();
 		if (pos >= valid) {
 			return 0;
 		}
 		return buffer[pos++];
 	}
-	bool BufferContainsNull() {
+	bool BufferContainsNull() noexcept {
 		EnsureData();
-		for (size_t i = 0;i < valid;i++) {
+		for (size_t i = 0; i < valid; i++) {
 			if (buffer[i] == '\0')
 				return true;
 		}
@@ -1458,12 +1469,12 @@ static bool IsWordCharacter(int ch) noexcept {
 }
 
 bool SciTEBase::GrepIntoDirectory(const FilePath &directory) {
-    const GUI::gui_char *sDirectory = directory.AsInternal();
+	const GUI::gui_char *sDirectory = directory.AsInternal();
 #ifdef __APPLE__
-    if (strcmp(sDirectory, "build") == 0)
-        return false;
+	if (strcmp(sDirectory, "build") == 0)
+		return false;
 #endif
-    return sDirectory[0] != '.';
+	return sDirectory[0] != '.';
 }
 
 void SciTEBase::GrepRecursive(GrepFlags gf, const FilePath &baseDir, const char *searchString, const GUI::gui_char *fileTypes) {
@@ -1490,7 +1501,7 @@ void SciTEBase::GrepRecursive(GrepFlags gf, const FilePath &baseDir, const char
 							const char *lineEnd = line + strlen(line);
 							while (match) {
 								if (((match == line) || !IsWordCharacter(match[-1])) &&
-								        ((match + searchLength == (lineEnd)) || !IsWordCharacter(match[searchLength]))) {
+										((match + searchLength == (lineEnd)) || !IsWordCharacter(match[searchLength]))) {
 									break;
 								}
 								match = strstr(match + 1, searchString);
@@ -1524,7 +1535,7 @@ void SciTEBase::GrepRecursive(GrepFlags gf, const FilePath &baseDir, const char
 	}
 }
 
-void SciTEBase::InternalGrep(GrepFlags gf, const GUI::gui_char *directory, const GUI::gui_char *fileTypes, const char *search, sptr_t &originalEnd) {
+void SciTEBase::InternalGrep(GrepFlags gf, const GUI::gui_char *directory, const GUI::gui_char *fileTypes, const char *search, SA::Position &originalEnd) {
 	GUI::ElapsedTime commandTime;
 	if (!(gf & grepStdOut)) {
 		std::string os;
