@@ -462,6 +462,7 @@ static const char *propertiesToForward[] = {
 	"fold.d.explicit.end",
 	"fold.d.explicit.start",
 	"fold.d.syntax.based",
+	"fold.dataflex.compilerlist",
 	"fold.directive",
 	"fold.haskell.imports",
 	"fold.html",
@@ -1198,7 +1199,7 @@ void SciTEBase::ReadProperties() {
 	}
 
 	// Create a margin column for the folding symbols
-	wEditor.SetMarginTypeN(2, static_cast<int>(SA::MarginType::Symbol));
+	wEditor.SetMarginTypeN(2, SA::MarginType::Symbol);
 
 	foldMarginWidth = props.GetInt("fold.margin.width");
 	if (foldMarginWidth == 0)
