@@ -11,8 +11,8 @@
  * Flash the given window for the asked @a duration to visually warn the user.
  */
 static void FlashThisWindow(
-    HWND hWnd,    		///< Window to flash handle.
-    int duration) {	///< Duration of the flash state.
+	HWND hWnd,    		///< Window to flash handle.
+	int duration) noexcept {	///< Duration of the flash state.
 
 	HDC hDC = ::GetDC(hWnd);
 	if (hDC) {
@@ -29,15 +29,13 @@ static void FlashThisWindow(
  * Play the given sound, loading if needed the corresponding DLL function.
  */
 static void PlayThisSound(
-    const char *sound,    	///< Path to a .wav file or string with a frequency value.
-    int duration,    		///< If @a sound is a frequency, gives the duration of the sound.
-    HMODULE &hMM) {		///< Multimedia DLL handle.
+	const char *sound,    	///< Path to a .wav file or string with a frequency value.
+	int duration,    		///< If @a sound is a frequency, gives the duration of the sound.
+	HMODULE &hMM) noexcept {		///< Multimedia DLL handle.
 
 	BOOL bPlayOK = false;
-	int soundFreq;
-	if (!sound || *sound == '\0') {
-		soundFreq = -1;	// No sound at all
-	} else {
+	int soundFreq = -1;	// Default is no sound at all
+	if (sound && *sound) {
 		soundFreq = atoi(sound);	// May be a frequency, not a filename
 	}
 
@@ -48,7 +46,7 @@ static void PlayThisSound(
 		}
 
 		if (hMM) {
-			typedef BOOL (WINAPI *MMFn) (LPCSTR, HMODULE, DWORD);
+			typedef BOOL (WINAPI *MMFn)(LPCSTR, HMODULE, DWORD);
 			MMFn fnMM = reinterpret_cast<MMFn>(::GetProcAddress(hMM, "PlaySoundA"));
 			if (fnMM) {
 				bPlayOK = fnMM(sound, NULL, SND_ASYNC | SND_FILENAME);
@@ -73,16 +71,18 @@ static void PlayThisSound(
 	}
 }
 
-static SciTEWin *Caller(HWND hDlg, UINT message, LPARAM lParam) {
+static SciTEWin *Caller(HWND hDlg, UINT message, LPARAM lParam) noexcept {
 	if (message == WM_INITDIALOG) {
 		::SetWindowLongPtr(hDlg, DWLP_USER, lParam);
 	}
-	return reinterpret_cast<SciTEWin*>(::GetWindowLongPtr(hDlg, DWLP_USER));
+	return reinterpret_cast<SciTEWin *>(::GetWindowLongPtr(hDlg, DWLP_USER));
 }
 
 void SciTEWin::WarnUser(int warnID) {
 	std::string warning;
-	char flashDuration[10], sound[_MAX_PATH], soundDuration[10];
+	char flashDuration[10] = "";
+	char sound[_MAX_PATH] = "";
+	char soundDuration[10] = "";
 
 	switch (warnID) {
 	case warnFindWrapped:
@@ -119,7 +119,7 @@ void SciTEWin::WarnUser(int warnID) {
 	PlayThisSound(sound, atoi(soundDuration), hMM);
 }
 
-bool SciTEWin::DialogHandled(GUI::WindowID id, MSG *pmsg) {
+bool SciTEWin::DialogHandled(GUI::WindowID id, MSG *pmsg) noexcept {
 	if (id) {
 		if (::IsDialogMessageW(static_cast<HWND>(id), pmsg))
 			return true;
@@ -138,11 +138,11 @@ bool SciTEWin::ModelessHandler(MSG *pmsg) {
 		// Allow commands, such as Ctrl+1 to be active while the Parameters dialog is
 		// visible so that a group of commands can be easily run with differing parameters.
 		const bool menuKey = (pmsg->message == WM_KEYDOWN) &&
-		               (pmsg->wParam != VK_TAB) &&
-		               (pmsg->wParam != VK_ESCAPE) &&
-		               (pmsg->wParam != VK_RETURN) &&
-		               (pmsg->wParam < 'A' || pmsg->wParam > 'Z') &&
-		               (IsKeyDown(VK_CONTROL) || !IsKeyDown(VK_MENU));
+				     (pmsg->wParam != VK_TAB) &&
+				     (pmsg->wParam != VK_ESCAPE) &&
+				     (pmsg->wParam != VK_RETURN) &&
+				     (pmsg->wParam < 'A' || pmsg->wParam > 'Z') &&
+				     (IsKeyDown(VK_CONTROL) || !IsKeyDown(VK_MENU));
 		if (!menuKey && DialogHandled(wParameters.GetID(), pmsg))
 			return true;
 	}
@@ -170,7 +170,7 @@ bool SciTEWin::ModelessHandler(MSG *pmsg) {
 //  DoDialog is a bit like something in PC Magazine May 28, 1991, page 357
 int SciTEWin::DoDialog(const TCHAR *resName, DLGPROC lpProc) {
 	const int result = static_cast<int>(
-		::DialogBoxParam(hInstance, resName, MainHWND(), lpProc, reinterpret_cast<LPARAM>(this)));
+				   ::DialogBoxParam(hInstance, resName, MainHWND(), lpProc, reinterpret_cast<LPARAM>(this)));
 
 	if (result == -1) {
 		GUI::gui_string errorNum = GUI::StringFromInteger(::GetLastError());
@@ -185,10 +185,10 @@ int SciTEWin::DoDialog(const TCHAR *resName, DLGPROC lpProc) {
 
 HWND SciTEWin::CreateParameterisedDialog(LPCWSTR lpTemplateName, DLGPROC lpProc) {
 	return ::CreateDialogParamW(hInstance,
-		lpTemplateName,
-		MainHWND(),
-		lpProc,
-		reinterpret_cast<LPARAM>(this));
+				    lpTemplateName,
+				    MainHWND(),
+				    lpProc,
+				    reinterpret_cast<LPARAM>(this));
 }
 
 GUI::gui_string SciTEWin::DialogFilterFromProperty(const GUI::gui_char *filterProperty) {
@@ -241,7 +241,7 @@ bool SciTEWin::OpenDialog(const FilePath &directory, const GUI::gui_char *filesF
 	GUI::gui_char openName[maxBufferSize]; // maximum common dialog buffer size (says mfc..)
 	openName[0] = '\0';
 
-	OPENFILENAMEW ofn = OPENFILENAMEW();
+	OPENFILENAMEW ofn {};
 	ofn.lStructSize = sizeof(ofn);
 	ofn.hwndOwner = MainHWND();
 	ofn.hInstance = hInstance;
@@ -260,10 +260,10 @@ bool SciTEWin::OpenDialog(const FilePath &directory, const GUI::gui_char *filesF
 
 	if (buffers.size() > 1) {
 		ofn.Flags |=
-		    OFN_EXPLORER |
-		    OFN_PATHMUSTEXIST |
-		    OFN_NOCHANGEDIR |
-		    OFN_ALLOWMULTISELECT;
+			OFN_EXPLORER |
+			OFN_PATHMUSTEXIST |
+			OFN_NOCHANGEDIR |
+			OFN_ALLOWMULTISELECT;
 	}
 	if (::GetOpenFileNameW(&ofn)) {
 		succeeded = true;
@@ -295,7 +295,7 @@ FilePath SciTEWin::ChooseSaveName(const FilePath &directory, const char *title,
 		if (!savePath.IsUntitled()) {
 			StringCopy(saveName, savePath.AsInternal());
 		}
-		OPENFILENAMEW ofn = OPENFILENAMEW();
+		OPENFILENAMEW ofn {};
 		ofn.lStructSize = sizeof(ofn);
 		ofn.hwndOwner = MainHWND();
 		ofn.hInstance = hInstance;
@@ -320,7 +320,7 @@ FilePath SciTEWin::ChooseSaveName(const FilePath &directory, const char *title,
 
 bool SciTEWin::SaveAsDialog() {
 	GUI::gui_string saveFilter = DialogFilterFromProperty(
-		GUI::StringFromUTF8(props.GetExpandedString("save.filter")).c_str());
+					     GUI::StringFromUTF8(props.GetExpandedString("save.filter")).c_str());
 	FilePath path = ChooseSaveName(filePath.Directory(), "Save File", saveFilter.c_str());
 	if (path.IsSet()) {
 		return SaveIfNotOpen(path, false);
@@ -337,7 +337,7 @@ void SciTEWin::SaveACopy() {
 
 void SciTEWin::SaveAsHTML() {
 	FilePath path = ChooseSaveName(filePath.Directory(), "Export File As HTML",
-	                              GUI_TEXT("Web (.html;.htm)\0*.html;*.htm\0"), ".html");
+				       GUI_TEXT("Web (.html;.htm)\0*.html;*.htm\0"), ".html");
 	if (path.IsSet()) {
 		SaveToHTML(path);
 	}
@@ -345,7 +345,7 @@ void SciTEWin::SaveAsHTML() {
 
 void SciTEWin::SaveAsRTF() {
 	FilePath path = ChooseSaveName(filePath.Directory(), "Export File As RTF",
-	                              GUI_TEXT("RTF (.rtf)\0*.rtf\0"), ".rtf");
+				       GUI_TEXT("RTF (.rtf)\0*.rtf\0"), ".rtf");
 	if (path.IsSet()) {
 		SaveToRTF(path);
 	}
@@ -353,7 +353,7 @@ void SciTEWin::SaveAsRTF() {
 
 void SciTEWin::SaveAsPDF() {
 	FilePath path = ChooseSaveName(filePath.Directory(), "Export File As PDF",
-	                              GUI_TEXT("PDF (.pdf)\0*.pdf\0"), ".pdf");
+				       GUI_TEXT("PDF (.pdf)\0*.pdf\0"), ".pdf");
 	if (path.IsSet()) {
 		SaveToPDF(path);
 	}
@@ -361,7 +361,7 @@ void SciTEWin::SaveAsPDF() {
 
 void SciTEWin::SaveAsTEX() {
 	FilePath path = ChooseSaveName(filePath.Directory(), "Export File As LaTeX",
-	                              GUI_TEXT("TeX (.tex)\0*.tex\0"), ".tex");
+				       GUI_TEXT("TeX (.tex)\0*.tex\0"), ".tex");
 	if (path.IsSet()) {
 		SaveToTEX(path);
 	}
@@ -369,7 +369,7 @@ void SciTEWin::SaveAsTEX() {
 
 void SciTEWin::SaveAsXML() {
 	FilePath path = ChooseSaveName(filePath.Directory(), "Export File As XML",
-	                              GUI_TEXT("XML (.xml)\0*.xml\0"), ".xml");
+				       GUI_TEXT("XML (.xml)\0*.xml\0"), ".xml");
 	if (path.IsSet()) {
 		SaveToXML(path);
 	}
@@ -377,7 +377,7 @@ void SciTEWin::SaveAsXML() {
 
 void SciTEWin::LoadSessionDialog() {
 	GUI::gui_char openName[MAX_PATH] = GUI_TEXT("");
-	OPENFILENAMEW ofn = OPENFILENAMEW();
+	OPENFILENAMEW ofn = {};
 	ofn.lStructSize = sizeof(ofn);
 	ofn.hwndOwner = MainHWND();
 	ofn.hInstance = hInstance;
@@ -398,7 +398,7 @@ void SciTEWin::LoadSessionDialog() {
 void SciTEWin::SaveSessionDialog() {
 	GUI::gui_char saveName[MAX_PATH] = GUI_TEXT("\0");
 	StringCopy(saveName, GUI_TEXT("SciTE.session"));
-	OPENFILENAMEW ofn = OPENFILENAMEW();
+	OPENFILENAMEW ofn = {};
 	ofn.lStructSize = sizeof(ofn);
 	ofn.hwndOwner = MainHWND();
 	ofn.hInstance = hInstance;
@@ -416,10 +416,10 @@ void SciTEWin::SaveSessionDialog() {
 	}
 }
 
-static void DeleteFontObject(HFONT &font) {
+static void DeleteFontObject(HFONT &font) noexcept {
 	if (font) {
 		::DeleteObject(font);
-		font = 0;
+		font = {};
 	}
 }
 
@@ -429,12 +429,11 @@ static void DeleteFontObject(HFONT &font) {
  * If OK, print the user choice, with optionally defined header and footer.
  */
 void SciTEWin::Print(
-    bool showDialog) {	///< false if must print silently (using default settings).
+	bool showDialog) {	///< false if must print silently (using default settings).
 
 	RemoveFindMarks();
-	PRINTDLG pdlg = {
-	                    sizeof(PRINTDLG), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
-	                };
+	PRINTDLG pdlg = {};
+	pdlg.lStructSize = sizeof(PRINTDLG);
 	pdlg.hwndOwner = MainHWND();
 	pdlg.hInstance = hInstance;
 	pdlg.Flags = PD_USEDEVMODECOPIES | PD_ALLPAGES | PD_RETURNDC;
@@ -444,14 +443,16 @@ void SciTEWin::Print(
 	pdlg.nMaxPage = 0xffffU; // We do not know how many pages in the
 	// document until the printer is selected and the paper size is known.
 	pdlg.nCopies = 1;
-	pdlg.hDC = 0;
+	pdlg.hDC = {};
 	pdlg.hDevMode = hDevMode;
 	pdlg.hDevNames = hDevNames;
 
+	// This code will not work for documents > 2GB
+
 	// See if a range has been selected
-	const Sci_CharacterRange crange = GetSelection();
-	const int startPos = static_cast<int>(crange.cpMin);
-	const int endPos = static_cast<int>(crange.cpMax);
+	const SA::Range rangeSelection = GetSelection();
+	const LONG startPos = static_cast<LONG>(rangeSelection.start);
+	const LONG endPos = static_cast<LONG>(rangeSelection.end);
 
 	if (startPos == endPos) {
 		pdlg.Flags |= PD_NOSELECTION;
@@ -493,19 +494,19 @@ void SciTEWin::Print(
 	// we take the entire width and height of the paper and
 	// subtract everything else.
 	rectPhysMargins.right = ptPage.x						// total paper width
-	                        - GetDeviceCaps(hdc, HORZRES) // printable width
-	                        - rectPhysMargins.left;				// left unprintable margin
+				- GetDeviceCaps(hdc, HORZRES) // printable width
+				- rectPhysMargins.left;				// left unprintable margin
 
 	rectPhysMargins.bottom = ptPage.y						// total paper height
-	                         - GetDeviceCaps(hdc, VERTRES)	// printable height
-	                         - rectPhysMargins.top;				// right unprintable margin
+				 - GetDeviceCaps(hdc, VERTRES)	// printable height
+				 - rectPhysMargins.top;				// right unprintable margin
 
 	// At this point, rectPhysMargins contains the widths of the
 	// unprintable regions on all four sides of the page in device units.
 
 	// Take in account the page setup given by the user (if one value is not null)
 	if (pagesetupMargin.left != 0 || pagesetupMargin.right != 0 ||
-	        pagesetupMargin.top != 0 || pagesetupMargin.bottom != 0) {
+			pagesetupMargin.top != 0 || pagesetupMargin.bottom != 0) {
 		GUI::Rectangle rectSetup;
 
 		// Convert the hundredths of millimeters (HiMetric) or
@@ -517,8 +518,8 @@ void SciTEWin::Print(
 		GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_IMEASURE, localeInfo, 3);
 
 		if (localeInfo[0] == '0') {	// Metric system. '1' is US System
-			rectSetup.left = MulDiv (pagesetupMargin.left, ptDpi.x, 2540);
-			rectSetup.top = MulDiv (pagesetupMargin.top, ptDpi.y, 2540);
+			rectSetup.left = MulDiv(pagesetupMargin.left, ptDpi.x, 2540);
+			rectSetup.top = MulDiv(pagesetupMargin.top, ptDpi.y, 2540);
 			rectSetup.right	= MulDiv(pagesetupMargin.right, ptDpi.x, 2540);
 			rectSetup.bottom	= MulDiv(pagesetupMargin.bottom, ptDpi.y, 2540);
 		} else {
@@ -553,22 +554,22 @@ void SciTEWin::Print(
 	std::string headerFormat = props.GetString("print.header.format");
 	std::string footerFormat = props.GetString("print.footer.format");
 
-	TEXTMETRIC tm;
+	TEXTMETRIC tm {};
 
 	std::string headerStyle = props.GetString("print.header.style");
 	StyleDefinition sdHeader(headerStyle.c_str());
 
 	int headerLineHeight = ::MulDiv(
-	                           (sdHeader.specified & StyleDefinition::sdSize) ? sdHeader.size : 9,
-	                           ptDpi.y, 72);
+				       (sdHeader.specified & StyleDefinition::sdSize) ? sdHeader.size : 9,
+				       ptDpi.y, 72);
 	HFONT fontHeader = ::CreateFontA(headerLineHeight,
-	                                0, 0, 0,
-	                                sdHeader.weight,
-	                                sdHeader.italics,
-	                                sdHeader.underlined,
-	                                0, 0, 0,
-	                                0, 0, 0,
-	                                (sdHeader.specified & StyleDefinition::sdFont) ? sdHeader.font.c_str() : "Arial");
+					 0, 0, 0,
+					 static_cast<int>(sdHeader.weight),
+					 sdHeader.italics,
+					 sdHeader.underlined,
+					 0, 0, 0,
+					 0, 0, 0,
+					 (sdHeader.specified & StyleDefinition::sdFont) ? sdHeader.font.c_str() : "Arial");
 	::SelectObject(hdc, fontHeader);
 	::GetTextMetrics(hdc, &tm);
 	headerLineHeight = tm.tmHeight + tm.tmExternalLeading;
@@ -577,24 +578,25 @@ void SciTEWin::Print(
 	StyleDefinition sdFooter(footerStyle.c_str());
 
 	int footerLineHeight = ::MulDiv(
-	                           (sdFooter.specified & StyleDefinition::sdSize) ? sdFooter.size : 9,
-	                           ptDpi.y, 72);
+				       (sdFooter.specified & StyleDefinition::sdSize) ? sdFooter.size : 9,
+				       ptDpi.y, 72);
 	HFONT fontFooter = ::CreateFontA(footerLineHeight,
-	                                0, 0, 0,
-	                                sdFooter.weight,
-	                                sdFooter.italics,
-	                                sdFooter.underlined,
-	                                0, 0, 0,
-	                                0, 0, 0,
-	                                (sdFooter.specified & StyleDefinition::sdFont) ? sdFooter.font.c_str() : "Arial");
+					 0, 0, 0,
+					 static_cast<int>(sdFooter.weight),
+					 sdFooter.italics,
+					 sdFooter.underlined,
+					 0, 0, 0,
+					 0, 0, 0,
+					 (sdFooter.specified & StyleDefinition::sdFont) ? sdFooter.font.c_str() : "Arial");
 	::SelectObject(hdc, fontFooter);
 	::GetTextMetrics(hdc, &tm);
 	footerLineHeight = tm.tmHeight + tm.tmExternalLeading;
 
-	DOCINFO di = {sizeof(DOCINFO), 0, 0, 0, 0};
+	DOCINFO di = {};
+	di.cbSize = sizeof(DOCINFO);
 	di.lpszDocName = windowName.c_str();
-	di.lpszOutput = 0;
-	di.lpszDatatype = 0;
+	di.lpszOutput = nullptr;
+	di.lpszDatatype = nullptr;
 	di.fwType = 0;
 	if (::StartDoc(hdc, &di) < 0) {
 		::DeleteDC(hdc);
@@ -605,7 +607,7 @@ void SciTEWin::Print(
 		return;
 	}
 
-	LONG lengthDoc = wEditor.Call(SCI_GETLENGTH);
+	LONG lengthDoc = static_cast<LONG>(wEditor.Length());
 	const LONG lengthDocMax = lengthDoc;
 	LONG lengthPrinted = 0;
 
@@ -651,7 +653,7 @@ void SciTEWin::Print(
 
 	while (lengthPrinted < lengthDoc) {
 		const bool printPage = (!(pdlg.Flags & PD_PAGENUMS) ||
-		             ((pageNum >= pdlg.nFromPage) && (pageNum <= pdlg.nToPage)));
+					((pageNum >= pdlg.nFromPage) && (pageNum <= pdlg.nToPage)));
 
 		char pageString[32];
 		sprintf(pageString, "%0d", pageNum);
@@ -662,20 +664,21 @@ void SciTEWin::Print(
 
 			if (headerFormat.size()) {
 				GUI::gui_string sHeader = GUI::StringFromUTF8(propsPrint.GetExpandedString("print.header.format"));
-				::SetTextColor(hdc, sdHeader.ForeAsLong());
-				::SetBkColor(hdc, sdHeader.BackAsLong());
+				::SetTextColor(hdc, sdHeader.Fore());
+				::SetBkColor(hdc, sdHeader.Back());
 				::SelectObject(hdc, fontHeader);
 				const UINT ta = ::SetTextAlign(hdc, TA_BOTTOM);
 				RECT rcw = {frPrint.rc.left, frPrint.rc.top - headerLineHeight - headerLineHeight / 2,
-				            frPrint.rc.right, frPrint.rc.top - headerLineHeight / 2};
+					    frPrint.rc.right, frPrint.rc.top - headerLineHeight / 2
+					   };
 				rcw.bottom = rcw.top + headerLineHeight;
 				::ExtTextOutW(hdc, frPrint.rc.left + 5, frPrint.rc.top - headerLineHeight / 2,
-				             ETO_OPAQUE, &rcw, sHeader.c_str(),
-				             static_cast<int>(sHeader.length()), NULL);
+					      ETO_OPAQUE, &rcw, sHeader.c_str(),
+					      static_cast<int>(sHeader.length()), nullptr);
 				::SetTextAlign(hdc, ta);
-				HPEN pen = ::CreatePen(0, 1, sdHeader.ForeAsLong());
+				HPEN pen = ::CreatePen(0, 1, sdHeader.Fore());
 				HPEN penOld = SelectPen(hdc, pen);
-				::MoveToEx(hdc, frPrint.rc.left, frPrint.rc.top - headerLineHeight / 4, NULL);
+				::MoveToEx(hdc, frPrint.rc.left, frPrint.rc.top - headerLineHeight / 4, nullptr);
 				::LineTo(hdc, frPrint.rc.right, frPrint.rc.top - headerLineHeight / 4);
 				SelectPen(hdc, penOld);
 				DeletePen(pen);
@@ -685,27 +688,26 @@ void SciTEWin::Print(
 		frPrint.chrg.cpMin = lengthPrinted;
 		frPrint.chrg.cpMax = lengthDoc;
 
-		lengthPrinted = wEditor.CallPointer(SCI_FORMATRANGE,
-		                           printPage,
-		                           &frPrint);
+		lengthPrinted = static_cast<LONG>(wEditor.FormatRange(printPage, &frPrint));
 
 		if (printPage) {
 			if (footerFormat.size()) {
 				GUI::gui_string sFooter = GUI::StringFromUTF8(propsPrint.GetExpandedString("print.footer.format"));
-				::SetTextColor(hdc, sdFooter.ForeAsLong());
-				::SetBkColor(hdc, sdFooter.BackAsLong());
+				::SetTextColor(hdc, sdFooter.Fore());
+				::SetBkColor(hdc, sdFooter.Back());
 				::SelectObject(hdc, fontFooter);
 				const UINT ta = ::SetTextAlign(hdc, TA_TOP);
 				RECT rcw = {frPrint.rc.left, frPrint.rc.bottom + footerLineHeight / 2,
-				            frPrint.rc.right, frPrint.rc.bottom + footerLineHeight + footerLineHeight / 2};
+					    frPrint.rc.right, frPrint.rc.bottom + footerLineHeight + footerLineHeight / 2
+					   };
 				::ExtTextOutW(hdc, frPrint.rc.left + 5, frPrint.rc.bottom + footerLineHeight / 2,
-				             ETO_OPAQUE, &rcw, sFooter.c_str(),
-				             static_cast<int>(sFooter.length()), NULL);
+					      ETO_OPAQUE, &rcw, sFooter.c_str(),
+					      static_cast<int>(sFooter.length()), nullptr);
 				::SetTextAlign(hdc, ta);
-				HPEN pen = ::CreatePen(0, 1, sdFooter.ForeAsLong());
+				HPEN pen = ::CreatePen(0, 1, sdFooter.Fore());
 				HPEN penOld = SelectPen(hdc, pen);
-				::SetBkColor(hdc, sdFooter.ForeAsLong());
-				::MoveToEx(hdc, frPrint.rc.left, frPrint.rc.bottom + footerLineHeight / 4, NULL);
+				::SetBkColor(hdc, sdFooter.Fore());
+				::MoveToEx(hdc, frPrint.rc.left, frPrint.rc.bottom + footerLineHeight / 4, nullptr);
 				::LineTo(hdc, frPrint.rc.right, frPrint.rc.bottom + footerLineHeight / 4);
 				SelectPen(hdc, penOld);
 				DeletePen(pen);
@@ -719,7 +721,7 @@ void SciTEWin::Print(
 			break;
 	}
 
-	wEditor.Call(SCI_FORMATRANGE, FALSE, 0);
+	wEditor.FormatRange(false, nullptr);
 
 	::EndDoc(hdc);
 	::DeleteDC(hdc);
@@ -728,15 +730,13 @@ void SciTEWin::Print(
 }
 
 void SciTEWin::PrintSetup() {
-	PAGESETUPDLG pdlg = {
-	                        sizeof(PAGESETUPDLG), 0, 0, 0, 0, {0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0}, 0, 0, 0, 0, 0, 0
-	                    };
-
+	PAGESETUPDLG pdlg = {};
+	pdlg.lStructSize = sizeof(PAGESETUPDLG);
 	pdlg.hwndOwner = MainHWND();
 	pdlg.hInstance = hInstance;
 
 	if (pagesetupMargin.left != 0 || pagesetupMargin.right != 0 ||
-	        pagesetupMargin.top != 0 || pagesetupMargin.bottom != 0) {
+			pagesetupMargin.top != 0 || pagesetupMargin.bottom != 0) {
 		pdlg.Flags = PSD_MARGINS;
 
 		pdlg.rtMargin.left = pagesetupMargin.left;
@@ -766,14 +766,14 @@ class Dialog {
 	HWND hDlg;
 public:
 
-	explicit Dialog(HWND hDlg_) : hDlg(hDlg_) {
+	explicit Dialog(HWND hDlg_) noexcept : hDlg(hDlg_) {
 	}
 
-	HWND Item(int id) {
+	HWND Item(int id) noexcept {
 		return ::GetDlgItem(hDlg, id);
 	}
 
-	void Enable(int id, bool enable) {
+	void Enable(int id, bool enable) noexcept {
 		::EnableWindow(Item(id), enable);
 	}
 
@@ -781,10 +781,14 @@ public:
 		return TextOfWindow(Item(id));
 	}
 
-	void SetItemText(int id, const GUI::gui_char *s) {
+	void SetItemText(int id, const GUI::gui_char *s) noexcept {
 		::SetDlgItemTextW(hDlg, id, s);
 	}
 
+	void SetItemText(int id, const GUI::gui_string &s) noexcept {
+		SetItemText(id, s.c_str());
+	}
+
 	// Handle Unicode controls (assume strings to be UTF-8 on Windows NT)
 
 	std::string ItemTextU(int id) {
@@ -792,14 +796,23 @@ public:
 	}
 
 	void SetItemTextU(int id, const std::string &s) {
-		SetItemText(id, GUI::StringFromUTF8(s).c_str());
+		SetItemText(id, GUI::StringFromUTF8(s));
 	}
 
-	void SetCheck(int id, bool value) {
+	std::optional<intptr_t> ItemInteger(int id) {
+		const std::string sValue = ItemTextU(id);
+		try {
+			return static_cast<intptr_t>(std::stoll(sValue));
+		} catch (...) {
+			return std::nullopt;
+		}
+	}
+
+	void SetCheck(int id, bool value) noexcept {
 		Button_SetCheck(Item(id), value ? BST_CHECKED : BST_UNCHECKED);
 	}
 
-	bool Checked(int id) {
+	bool Checked(int id) noexcept {
 		return BST_CHECKED == Button_GetCheck(Item(id));
 	}
 
@@ -808,7 +821,7 @@ public:
 		ComboBox_ResetContent(combo);
 		for (int i = 0; i < mem.Length(); i++) {
 			GUI::gui_string gs = GUI::StringFromUTF8(mem.At(i));
-			ComboBox_AddString(combo, gs.c_str());
+			ComboBoxAppend(combo, gs);
 		}
 		if (useTop) {
 			ComboBox_SetCurSel(combo, 0);
@@ -818,14 +831,14 @@ public:
 };
 
 static void FillComboFromProps(HWND combo, PropSetFile &props) {
-	const char *key;
-	const char *val;
+	const char *key = nullptr;
+	const char *val = nullptr;
 	if (props.GetFirst(key, val)) {
 		GUI::gui_string wkey = GUI::StringFromUTF8(key);
-		ComboBox_AddString(combo, wkey.c_str());
+		ComboBoxAppend(combo, wkey);
 		while (props.GetNext(key, val)) {
 			wkey = GUI::StringFromUTF8(key);
-			ComboBox_AddString(combo, wkey.c_str());
+			ComboBoxAppend(combo, wkey);
 		}
 	}
 }
@@ -869,7 +882,7 @@ void SciTEWin::ShowBackgroundProgress(const GUI::gui_string &explanation, size_t
 class DialogFindReplace : public Dialog, public SearchUI  {
 	bool advanced;
 public:
-	DialogFindReplace(HWND hDlg_, bool advanced_) :
+	DialogFindReplace(HWND hDlg_, bool advanced_) noexcept :
 		Dialog(hDlg_), advanced(advanced_) {
 	}
 	void GrabFields();
@@ -932,7 +945,8 @@ BOOL SciTEWin::FindMessage(HWND hDlg, UINT message, WPARAM wParam) {
 			dlg.SetCheck(IDFINDSTYLE, findInStyle);
 			dlg.Enable(IDFINDSTYLE, findInStyle);
 			Edit_LimitText(dlg.Item(IDFINDSTYLE), 3);
-			::SetDlgItemInt(hDlg, IDFINDSTYLE, wEditor.Call(SCI_GETSTYLEAT, wEditor.Call(SCI_GETCURRENTPOS)), FALSE);
+			dlg.SetItemText(IDFINDSTYLE, std::to_wstring(
+						wEditor.UnsignedStyleAt(wEditor.CurrentPos())));
 		}
 		return TRUE;
 
@@ -945,8 +959,8 @@ BOOL SciTEWin::FindMessage(HWND hDlg, UINT message, WPARAM wParam) {
 			::EndDialog(hDlg, IDCANCEL);
 			wFindReplace.Destroy();
 			return FALSE;
-		} else if ( (ControlIDOfWParam(wParam) == IDOK) ||
-		            (ControlIDOfWParam(wParam) == IDMARKALL) ) {
+		} else if ((ControlIDOfWParam(wParam) == IDOK) ||
+				(ControlIDOfWParam(wParam) == IDMARKALL)) {
 			dlg.GrabFields();
 			if (ControlIDOfWParam(wParam) == IDMARKALL) {
 				MarkAll(markWithBookMarks);
@@ -988,7 +1002,7 @@ BOOL SciTEWin::HandleReplaceCommand(int cmd, bool reverseDirection) {
 		dlg.GrabFields();
 	}
 
-	int replacements = 0;
+	intptr_t replacements = 0;
 	if (cmd == IDOK) {
 		FindNext(reverseDirection);
 		FillCombos(dlg);
@@ -1002,8 +1016,8 @@ BOOL SciTEWin::HandleReplaceCommand(int cmd, bool reverseDirection) {
 		replacements = ReplaceInBuffers();
 		FillCombos(dlg);
 	}
-	GUI::gui_string replDone = GUI::StringFromInteger(replacements);
-	dlg.SetItemText(IDREPLDONE, replDone.c_str());
+	GUI::gui_string replDone = std::to_wstring(replacements);
+	dlg.SetItemText(IDREPLDONE, replDone);
 
 	return TRUE;
 }
@@ -1022,7 +1036,8 @@ BOOL SciTEWin::ReplaceMessage(HWND hDlg, UINT message, WPARAM wParam) {
 		dlg.FillFields();
 		if (FindReplaceAdvanced()) {
 			dlg.Enable(IDFINDSTYLE, findInStyle);
-			::SetDlgItemInt(hDlg, IDFINDSTYLE, wEditor.Call(SCI_GETSTYLEAT, wEditor.Call(SCI_GETCURRENTPOS)), FALSE);
+			dlg.SetItemText(IDFINDSTYLE, std::to_wstring(
+						wEditor.UnsignedStyleAt(wEditor.CurrentPos())));
 		}
 		if (findWhat.length() != 0 && props.GetInt("find.replacewith.focus", 1)) {
 			::SetFocus(::GetDlgItem(hDlg, IDREPLACEWITH));
@@ -1172,7 +1187,7 @@ void SciTEWin::PerformGrep() {
 void SciTEWin::FillCombos(Dialog &dlg) {
 	dlg.FillComboFromMemory(IDFINDWHAT, memFinds, true);
 	if (dlg.Item(IDREPLACEWITH))    // Replace combo is presented
-		dlg.FillComboFromMemory(IDREPLACEWITH, memReplaces,true);
+		dlg.FillComboFromMemory(IDREPLACEWITH, memReplaces, true);
 }
 
 void SciTEWin::FillCombosForGrep(Dialog &dlg) {
@@ -1255,13 +1270,13 @@ BOOL SciTEWin::GrepMessage(HWND hDlg, UINT message, WPARAM wParam) {
 			// allocated by the shell. Eventually, we will need to free this
 			// memory, so we need to get a pointer to the shell malloc COM
 			// object that will free the PIDL later on.
-			LPMALLOC pShellMalloc = 0;
+			LPMALLOC pShellMalloc = nullptr;
 			if (::SHGetMalloc(&pShellMalloc) == NO_ERROR) {
 				// If we were able to get the shell malloc object,
 				// then proceed by initializing the BROWSEINFO stuct
-				BROWSEINFO info = BROWSEINFO();
+				BROWSEINFO info {};
 				info.hwndOwner = hDlg;
-				info.pidlRoot = NULL;
+				info.pidlRoot = nullptr;
 				TCHAR szDisplayName[MAX_PATH] = TEXT("");
 				info.pszDisplayName = szDisplayName;
 				GUI::gui_string title = localiser.Text("Select a folder to search from");
@@ -1371,24 +1386,26 @@ void SciTEWin::DestroyFindReplace() {
 }
 
 BOOL SciTEWin::GoLineMessage(HWND hDlg, UINT message, WPARAM wParam) {
+	Dialog dlg(hDlg);
+
 	switch (message) {
 
 	case WM_INITDIALOG: {
-			int position = wEditor.Call(SCI_GETCURRENTPOS);
-			const int lineNumber = wEditor.Call(SCI_LINEFROMPOSITION, position) + 1;
-			const int lineStart = wEditor.Call(SCI_POSITIONFROMLINE, lineNumber - 1);
+			SA::Position position = wEditor.CurrentPos();
+			const SA::Line lineNumber = wEditor.LineFromPosition(position) + 1;
+			const SA::Position lineStart = wEditor.LineStart(lineNumber - 1);
 			int characterOnLine = 1;
 			while (position > lineStart) {
-				position = wEditor.Call(SCI_POSITIONBEFORE, position);
+				position = wEditor.PositionBefore(position);
 				characterOnLine++;
 			}
 
 			LocaliseDialog(hDlg);
 			::SendDlgItemMessage(hDlg, IDGOLINE, EM_LIMITTEXT, 10, 1);
 			::SendDlgItemMessage(hDlg, IDGOLINECHAR, EM_LIMITTEXT, 10, 1);
-			::SetDlgItemInt(hDlg, IDCURRLINE, lineNumber, FALSE);
-			::SetDlgItemInt(hDlg, IDCURRLINECHAR, characterOnLine, FALSE);
-			::SetDlgItemInt(hDlg, IDLASTLINE, wEditor.Call(SCI_GETLINECOUNT), FALSE);
+			dlg.SetItemText(IDCURRLINE, std::to_wstring(lineNumber));
+			dlg.SetItemText(IDCURRLINECHAR, std::to_wstring(characterOnLine));
+			dlg.SetItemText(IDLASTLINE, std::to_wstring(wEditor.LineCount()));
 		}
 		return TRUE;
 
@@ -1401,29 +1418,26 @@ BOOL SciTEWin::GoLineMessage(HWND hDlg, UINT message, WPARAM wParam) {
 			::EndDialog(hDlg, IDCANCEL);
 			return FALSE;
 		} else if (ControlIDOfWParam(wParam) == IDOK) {
-			BOOL bHasLine;
-			int lineNumber = static_cast<int>(
-			                     ::GetDlgItemInt(hDlg, IDGOLINE, &bHasLine, FALSE));
-			BOOL bHasChar;
-			int characterOnLine = static_cast<int>(
-			                     ::GetDlgItemInt(hDlg, IDGOLINECHAR, &bHasChar, FALSE));
+			const std::optional<intptr_t> lineNumberOpt = dlg.ItemInteger(IDGOLINE);
+			const std::optional<intptr_t> characterOnLineOpt = dlg.ItemInteger(IDGOLINECHAR);
 
-			if (bHasLine || bHasChar) {
-				if (!bHasLine)
-					lineNumber = wEditor.Call(SCI_LINEFROMPOSITION, wEditor.Call(SCI_GETCURRENTPOS)) + 1;
+			if (lineNumberOpt || characterOnLineOpt) {
+				const intptr_t lineNumber = lineNumberOpt.value_or(
+								    wEditor.LineFromPosition(wEditor.CurrentPos()) + 1);
 
 				GotoLineEnsureVisible(lineNumber - 1);
 
-				if (bHasChar && characterOnLine > 1 && lineNumber <= wEditor.Call(SCI_GETLINECOUNT)) {
+				if (characterOnLineOpt && characterOnLineOpt.value() > 1 && lineNumber <= wEditor.LineCount()) {
 					// Constrain to the requested line
-					const int lineStart = wEditor.Call(SCI_POSITIONFROMLINE, lineNumber - 1);
-					const int lineEnd = wEditor.Call(SCI_GETLINEENDPOSITION, lineNumber - 1);
+					const SA::Position lineStart = wEditor.LineStart(lineNumber - 1);
+					const SA::Position lineEnd = wEditor.LineEnd(lineNumber - 1);
 
-					int position = lineStart;
+					SA::Position characterOnLine = characterOnLineOpt.value();
+					SA::Position position = lineStart;
 					while (--characterOnLine && position < lineEnd)
-						position = wEditor.Call(SCI_POSITIONAFTER, position);
+						position = wEditor.PositionAfter(position);
 
-					wEditor.Call(SCI_GOTOPOS, position);
+					wEditor.GotoPos(position);
 				}
 			}
 			::EndDialog(hDlg, IDOK);
@@ -1479,23 +1493,25 @@ bool SciTEWin::AbbrevDialog() {
 }
 
 BOOL SciTEWin::TabSizeMessage(HWND hDlg, UINT message, WPARAM wParam) {
+	Dialog dlg(hDlg);
+
 	switch (message) {
 
 	case WM_INITDIALOG: {
 			LocaliseDialog(hDlg);
 			::SendDlgItemMessage(hDlg, IDTABSIZE, EM_LIMITTEXT, 2, 1);
-			int tabSize = wEditor.Call(SCI_GETTABWIDTH);
+			int tabSize = wEditor.TabWidth();
 			if (tabSize > 99)
 				tabSize = 99;
-			::SetDlgItemInt(hDlg, IDTABSIZE, tabSize, FALSE);
+			dlg.SetItemText(IDTABSIZE, std::to_wstring(tabSize));
 
 			::SendDlgItemMessage(hDlg, IDINDENTSIZE, EM_LIMITTEXT, 2, 1);
-			int indentSize = wEditor.Call(SCI_GETINDENT);
+			int indentSize = wEditor.Indent();
 			if (indentSize > 99)
 				indentSize = 99;
-			::SetDlgItemInt(hDlg, IDINDENTSIZE, indentSize, FALSE);
+			dlg.SetItemText(IDINDENTSIZE, std::to_wstring(indentSize));
 
-			::CheckDlgButton(hDlg, IDUSETABS, wEditor.Call(SCI_GETUSETABS));
+			::CheckDlgButton(hDlg, IDUSETABS, wEditor.UseTabs());
 			return TRUE;
 		}
 
@@ -1508,16 +1524,16 @@ BOOL SciTEWin::TabSizeMessage(HWND hDlg, UINT message, WPARAM wParam) {
 			::EndDialog(hDlg, IDCANCEL);
 			return FALSE;
 		} else if ((ControlIDOfWParam(wParam) == IDCONVERT) ||
-			(ControlIDOfWParam(wParam) == IDOK)) {
+				(ControlIDOfWParam(wParam) == IDOK)) {
 			BOOL bOK;
-			const int tabSize = static_cast<int>(::GetDlgItemInt(hDlg, IDTABSIZE, &bOK, FALSE));
+			const int tabSize = ::GetDlgItemInt(hDlg, IDTABSIZE, &bOK, FALSE);
 			if (tabSize > 0)
-				wEditor.Call(SCI_SETTABWIDTH, tabSize);
-			const int indentSize = static_cast<int>(::GetDlgItemInt(hDlg, IDINDENTSIZE, &bOK, FALSE));
+				wEditor.SetTabWidth(tabSize);
+			const int indentSize = ::GetDlgItemInt(hDlg, IDINDENTSIZE, &bOK, FALSE);
 			if (indentSize > 0)
-				wEditor.Call(SCI_SETINDENT, indentSize);
-			const bool useTabs = static_cast<bool>(::IsDlgButtonChecked(hDlg, IDUSETABS));
-			wEditor.Call(SCI_SETUSETABS, useTabs);
+				wEditor.SetIndent(indentSize);
+			const bool useTabs = ::IsDlgButtonChecked(hDlg, IDUSETABS) == BST_CHECKED;
+			wEditor.SetUseTabs(useTabs);
 			if (ControlIDOfWParam(wParam) == IDCONVERT) {
 				ConvertIndentation(tabSize, useTabs);
 			}
@@ -1614,7 +1630,7 @@ bool SciTEWin::ParametersDialog(bool modal) {
 	modalParameters = modal;
 	if (modal) {
 		success = DoDialog(TEXT("PARAMETERS"), ParametersDlg) == IDOK;
-		wParameters = 0;
+		wParameters = NULL;
 	} else {
 		CreateParameterisedDialog(TEXT("PARAMETERSNONMODAL"), ParametersDlg);
 		wParameters.Show();
@@ -1667,13 +1683,13 @@ BOOL SciTEWin::AboutMessage(HWND hDlg, UINT message, WPARAM wParam) {
 	switch (message) {
 
 	case WM_INITDIALOG: {
-		LocaliseDialog(hDlg);
-		GUI::ScintillaWindow ss;
-		HWND hwndCredits = ::GetDlgItem(hDlg, IDABOUTSCINTILLA);
-		const LONG_PTR subclassedProc = ::SetWindowLongPtr(hwndCredits, GWLP_WNDPROC, reinterpret_cast<LONG_PTR>(CreditsWndProc));
-		::SetWindowLongPtr(hwndCredits, GWLP_USERDATA, subclassedProc);
-		ss.SetScintilla(hwndCredits);
-		SetAboutMessage(ss, staticBuild ? "Sc1  " : "SciTE");
+			LocaliseDialog(hDlg);
+			GUI::ScintillaWindow ss;
+			HWND hwndCredits = ::GetDlgItem(hDlg, IDABOUTSCINTILLA);
+			const LONG_PTR subclassedProc = ::SetWindowLongPtr(hwndCredits, GWLP_WNDPROC, reinterpret_cast<LONG_PTR>(CreditsWndProc));
+			::SetWindowLongPtr(hwndCredits, GWLP_USERDATA, subclassedProc);
+			ss.SetScintilla(hwndCredits);
+			SetAboutMessage(ss, staticBuild ? "Sc1  " : "SciTE");
 		}
 		return TRUE;
 
