@@ -7,15 +7,15 @@
 
 #include "SciTEWin.h"
 
-void *PointerFromWindow(HWND hWnd) {
+void *PointerFromWindow(HWND hWnd) noexcept {
 	return reinterpret_cast<void *>(::GetWindowLongPtr(hWnd, 0));
 }
 
-void SetWindowPointer(HWND hWnd, void *ptr) {
+void SetWindowPointer(HWND hWnd, void *ptr) noexcept {
 	::SetWindowLongPtr(hWnd, 0, reinterpret_cast<LONG_PTR>(ptr));
 }
 
-void *SetWindowPointerFromCreate(HWND hWnd, LPARAM lParam) {
+void *SetWindowPointerFromCreate(HWND hWnd, LPARAM lParam) noexcept {
 	LPCREATESTRUCT pcs = reinterpret_cast<LPCREATESTRUCT>(lParam);
 	void *ptr = pcs->lpCreateParams;
 	SetWindowPointer(hWnd, ptr);
@@ -43,32 +43,40 @@ GUI::gui_string ClassNameOfWindow(HWND hWnd) {
 		return GUI::gui_string();
 }
 
+void ComboBoxAppend(HWND hWnd, const GUI::gui_string &gs) noexcept {
+	ComboBox_AddString(hWnd, gs.c_str());
+}
+
 namespace {
 
-void SetFontHandle(const GUI::Window &w, HFONT hfont) {
+HINSTANCE ApplicationInstance() noexcept {
+	return ::GetModuleHandle(NULL);
+}
+
+void SetFontHandle(const GUI::Window &w, HFONT hfont) noexcept {
 	SetWindowFont(HwndOf(w), hfont, 0);
 }
 
-void CheckButton(const GUI::Window &wButton, bool checked) {
+void CheckButton(const GUI::Window &wButton, bool checked) noexcept {
 	Button_SetCheck(HwndOf(wButton), checked ? BST_CHECKED : BST_UNCHECKED);
 }
 
-SIZE SizeButton(const GUI::Window &wButton) {
+SIZE SizeButton(const GUI::Window &wButton) noexcept {
 	SIZE sz = { 0, 0 };
 #ifdef BCM_GETIDEALSIZE
 	// Push buttons can be measured with BCM_GETIDEALSIZE.
 	::SendMessage(HwndOf(wButton),
-		BCM_GETIDEALSIZE, 0, reinterpret_cast<LPARAM>(&sz));
+		      BCM_GETIDEALSIZE, 0, reinterpret_cast<LPARAM>(&sz));
 #else
 	(void)wButton;	// Avoid warning from GCC
 #endif
 	return sz;
 }
 
-int WidthText(HFONT hfont, const GUI::gui_char *text) {
+int WidthText(HFONT hfont, const GUI::gui_char *text) noexcept {
 	HDC hdcMeasure = ::CreateCompatibleDC(NULL);
 	HFONT hfontOriginal = SelectFont(hdcMeasure, hfont);
-	RECT rcText = {0,0, 2000, 2000};
+	RECT rcText = {0, 0, 2000, 2000};
 	::DrawText(hdcMeasure, text, -1, &rcText, DT_CALCRECT);
 	const int width = rcText.right - rcText.left;
 	SelectFont(hdcMeasure, hfontOriginal);
@@ -124,14 +132,14 @@ void SetComboFromMemory(GUI::Window w, const ComboMemory &mem) {
 	ComboBox_ResetContent(combo);
 	for (int i = 0; i < mem.Length(); i++) {
 		GUI::gui_string gs = GUI::StringFromUTF8(mem.At(i));
-		ComboBox_AddString(combo, gs.c_str());
+		ComboBoxAppend(combo, gs);
 	}
 }
 
 }
 
 LRESULT PASCAL BaseWin::StWndProc(
-    HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM lParam) {
+	HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM lParam) {
 	if (iMessage == WM_CREATE) {
 		// Pointer to BaseWin passed with WM_CREATE so remember in window pointer
 		BaseWin *basePassed = static_cast<BaseWin *>(SetWindowPointerFromCreate(hWnd, lParam));
@@ -171,9 +179,9 @@ GUI::Window Strip::CreateText(const char *text) {
 	const int width = WidthText(fontText, localised.c_str()) + 4;
 	GUI::Window w;
 	w.SetID(::CreateWindowEx(0, TEXT("Static"), localised.c_str(),
-		WS_CHILD | WS_CLIPSIBLINGS | SS_RIGHT,
-		2, 2, width, 21,
-		Hwnd(), HmenuID(0), ::GetModuleHandle(NULL), 0));
+				 WS_CHILD | WS_CLIPSIBLINGS | SS_RIGHT,
+				 2, 2, width, 21,
+				 Hwnd(), HmenuID(0), ::ApplicationInstance(), 0));
 	SetFontHandle(w, fontText);
 	w.Show();
 	return w;
@@ -181,26 +189,26 @@ GUI::Window Strip::CreateText(const char *text) {
 
 #define PACKVERSION(major,minor) MAKELONG(minor,major)
 
-static DWORD GetVersion(LPCTSTR lpszDllName) {
-    DWORD dwVersion = 0;
-    HINSTANCE hinstDll = ::LoadLibrary(lpszDllName);
-    if (hinstDll) {
-        DLLGETVERSIONPROC pDllGetVersion = reinterpret_cast<DLLGETVERSIONPROC>(
-			::GetProcAddress(hinstDll, "DllGetVersion"));
+static DWORD GetVersion(LPCTSTR lpszDllName) noexcept {
+	DWORD dwVersion = 0;
+	HINSTANCE hinstDll = ::LoadLibrary(lpszDllName);
+	if (hinstDll) {
+		DLLGETVERSIONPROC pDllGetVersion = reinterpret_cast<DLLGETVERSIONPROC>(
+				::GetProcAddress(hinstDll, "DllGetVersion"));
 
-        if (pDllGetVersion) {
-            DLLVERSIONINFO dvi;
-            ::ZeroMemory(&dvi, sizeof(dvi));
-            dvi.cbSize = sizeof(dvi);
+		if (pDllGetVersion) {
+			DLLVERSIONINFO dvi;
+			::ZeroMemory(&dvi, sizeof(dvi));
+			dvi.cbSize = sizeof(dvi);
 
-            const HRESULT hr = (*pDllGetVersion)(&dvi);
-            if (SUCCEEDED(hr)) {
-               dwVersion = PACKVERSION(dvi.dwMajorVersion, dvi.dwMinorVersion);
-            }
-        }
-        ::FreeLibrary(hinstDll);
-    }
-    return dwVersion;
+			const HRESULT hr = (*pDllGetVersion)(&dvi);
+			if (SUCCEEDED(hr)) {
+				dwVersion = PACKVERSION(dvi.dwMajorVersion, dvi.dwMinorVersion);
+			}
+		}
+		::FreeLibrary(hinstDll);
+	}
+	return dwVersion;
 }
 
 GUI::Window Strip::CreateButton(const char *text, size_t ident, bool check) {
@@ -235,54 +243,66 @@ GUI::Window Strip::CreateButton(const char *text, size_t ident, bool check) {
 	}
 	GUI::Window w;
 	w.SetID(::CreateWindowEx(0, TEXT("Button"), localised.c_str(),
-		WS_CHILD | WS_TABSTOP | WS_CLIPSIBLINGS |
-		(check ? (BS_AUTOCHECKBOX | BS_PUSHLIKE | BS_BITMAP) : BS_PUSHBUTTON),
-		2, 2, width, height,
-		Hwnd(), HmenuID(ident), ::GetModuleHandle(NULL), 0));
+				 WS_CHILD | WS_TABSTOP | WS_CLIPSIBLINGS |
+				 (check ? (BS_AUTOCHECKBOX | BS_PUSHLIKE | BS_BITMAP) : BS_PUSHBUTTON),
+				 2, 2, width, height,
+				 Hwnd(), HmenuID(ident), ::ApplicationInstance(), 0));
 	if (check) {
 		int resNum = IDBM_WORD;
 		switch (ident) {
-		case IDWHOLEWORD: resNum = IDBM_WORD; break;
-		case IDMATCHCASE: resNum = IDBM_CASE; break;
-		case IDREGEXP: resNum = IDBM_REGEX; break;
-		case IDUNSLASH: resNum = IDBM_BACKSLASH; break;
-		case IDWRAP: resNum = IDBM_AROUND; break;
-		case IDDIRECTIONUP: resNum = IDBM_UP; break;
+		case IDWHOLEWORD:
+			resNum = IDBM_WORD;
+			break;
+		case IDMATCHCASE:
+			resNum = IDBM_CASE;
+			break;
+		case IDREGEXP:
+			resNum = IDBM_REGEX;
+			break;
+		case IDUNSLASH:
+			resNum = IDBM_BACKSLASH;
+			break;
+		case IDWRAP:
+			resNum = IDBM_AROUND;
+			break;
+		case IDDIRECTIONUP:
+			resNum = IDBM_UP;
+			break;
 		}
 
-		const UINT flags = (GetVersion(TEXT("COMCTL32")) >= PACKVERSION(6,0)) ?
-			(LR_DEFAULTSIZE) : (LR_DEFAULTSIZE|LR_LOADMAP3DCOLORS);
+		const UINT flags = (GetVersion(TEXT("COMCTL32")) >= PACKVERSION(6, 0)) ?
+				   (LR_DEFAULTSIZE) : (LR_DEFAULTSIZE|LR_LOADMAP3DCOLORS);
 		HBITMAP bm = static_cast<HBITMAP>(::LoadImage(
-			::GetModuleHandle(NULL), MAKEINTRESOURCE(resNum + resDifference), IMAGE_BITMAP,
-			bmpDimension, bmpDimension, flags));
+				::ApplicationInstance(), MAKEINTRESOURCE(resNum + resDifference), IMAGE_BITMAP,
+				bmpDimension, bmpDimension, flags));
 
 		::SendMessage(HwndOf(w),
-			BM_SETIMAGE, IMAGE_BITMAP, reinterpret_cast<LPARAM>(bm));
+			      BM_SETIMAGE, IMAGE_BITMAP, reinterpret_cast<LPARAM>(bm));
 	}
 	SetFontHandle(w, fontText);
 	w.Show();
 	if (!check) {
 		const SIZE sz = SizeButton(w);
 		if (sz.cx > 0) {
-			const GUI::Rectangle rc(0,0, sz.cx + 2 * WidthText(fontText, TEXT(" ")), sz.cy);
+			const GUI::Rectangle rc(0, 0, sz.cx + 2 * WidthText(fontText, TEXT(" ")), sz.cy);
 			w.SetPosition(rc);
 		}
 	}
-	TOOLINFO toolInfo = TOOLINFO();
+	TOOLINFO toolInfo {};
 	toolInfo.cbSize = sizeof(toolInfo);
 	toolInfo.uFlags = TTF_SUBCLASS | TTF_IDISHWND;
-	toolInfo.hinst = ::GetModuleHandle(NULL);
+	toolInfo.hinst = ::ApplicationInstance();
 	toolInfo.hwnd = Hwnd();
 	toolInfo.uId = reinterpret_cast<UINT_PTR>(w.GetID());
 	toolInfo.lpszText = LPSTR_TEXTCALLBACK;
 	::GetClientRect(Hwnd(), &toolInfo.rect);
 	::SendMessageW(HwndOf(wToolTip), TTM_ADDTOOLW,
-		0, reinterpret_cast<LPARAM>(&toolInfo));
+		       0, reinterpret_cast<LPARAM>(&toolInfo));
 	::SendMessage(HwndOf(wToolTip), TTM_ACTIVATE, TRUE, 0);
 	return w;
 }
 
-void Strip::Tab(bool forwards) {
+void Strip::Tab(bool forwards) noexcept {
 	HWND wToFocus = ::GetNextDlgTabItem(Hwnd(), ::GetFocus(), !forwards);
 	if (wToFocus)
 		::SetFocus(wToFocus);
@@ -296,24 +316,24 @@ void Strip::Creation() {
 	fontText = ::CreateFontIndirect(&ncm.lfMessageFont);
 
 	wToolTip = ::CreateWindowEx(0,
-		TOOLTIPS_CLASSW, NULL,
-		WS_POPUP | TTS_ALWAYSTIP,
-		CW_USEDEFAULT, CW_USEDEFAULT,
-		CW_USEDEFAULT, CW_USEDEFAULT,
-		Hwnd(), NULL, ::GetModuleHandle(NULL), NULL);
+				    TOOLTIPS_CLASSW, NULL,
+				    WS_POPUP | TTS_ALWAYSTIP,
+				    CW_USEDEFAULT, CW_USEDEFAULT,
+				    CW_USEDEFAULT, CW_USEDEFAULT,
+				    Hwnd(), NULL, ::ApplicationInstance(), NULL);
 
 	SetTheme();
 }
 
-void Strip::Destruction() {
+void Strip::Destruction() noexcept {
 	if (fontText)
 		::DeleteObject(fontText);
-	fontText = 0;
+	fontText = {};
 #ifdef THEME_AVAILABLE
 	if (hTheme)
 		::CloseThemeData(hTheme);
 #endif
-	hTheme = 0;
+	hTheme = {};
 }
 
 void Strip::Close() {
@@ -373,7 +393,7 @@ void Strip::Size() {
 
 namespace {
 
-RECT RECTFromRectangle(GUI::Rectangle r) {
+RECT RECTFromRectangle(GUI::Rectangle r) noexcept {
 	RECT rc = { r.left, r.top, r.right, r.bottom };
 	return rc;
 }
@@ -387,7 +407,7 @@ void Strip::Paint(HDC hDC) {
 	::FillRect(hDC, &rc, hbrFace);
 	::DeleteObject(hbrFace);
 
-	if (HasClose()){
+	if (HasClose()) {
 		// Draw close box
 		RECT rcClose = RECTFromRectangle(CloseArea());
 		if (hTheme) {
@@ -399,7 +419,7 @@ void Strip::Paint(HDC hDC) {
 				closeAppearence = CBS_PUSHED;
 			}
 			::DrawThemeBackground(hTheme, hDC, WP_SMALLCLOSEBUTTON, closeAppearence,
-				&rcClose, NULL);
+					      &rcClose, nullptr);
 #endif
 		} else {
 			int closeAppearence = 0;
@@ -410,12 +430,12 @@ void Strip::Paint(HDC hDC) {
 			}
 
 			DrawFrameControl(hDC, &rcClose, DFC_CAPTION,
-				DFCS_CAPTIONCLOSE | closeAppearence);
+					 DFCS_CAPTIONCLOSE | closeAppearence);
 		}
 	}
 }
 
-bool Strip::HasClose() const {
+bool Strip::HasClose() const noexcept {
 	return true;
 }
 
@@ -428,14 +448,13 @@ GUI::Rectangle Strip::CloseArea() {
 		rcClose.bottom = rcClose.top + closeSize.cy;
 		return rcClose;
 	} else {
-		return GUI::Rectangle(-1,-1,-1,-1);
+		return GUI::Rectangle(-1, -1, -1, -1);
 	}
 }
 
 GUI::Rectangle Strip::LineArea(int line) {
 	GUI::Rectangle rcLine = GetPosition();
 
-	rcLine.bottom -= rcLine.top;
 	rcLine.right -= rcLine.left;
 
 	rcLine.left = space;
@@ -449,7 +468,7 @@ GUI::Rectangle Strip::LineArea(int line) {
 	return rcLine;
 }
 
-int Strip::Lines() const {
+int Strip::Lines() const noexcept {
 	return 1;
 }
 
@@ -487,7 +506,7 @@ void Strip::TrackMouse(GUI::Point pt) {
 		InvalidateClose();
 }
 
-void Strip::SetTheme() {
+void Strip::SetTheme() noexcept {
 #ifdef THEME_AVAILABLE
 	if (hTheme)
 		::CloseThemeData(hTheme);
@@ -499,7 +518,7 @@ void Strip::SetTheme() {
 		::ReleaseDC(Hwnd(), hdc);
 		space = scale * 2 / 96;
 		const HRESULT hr = ::GetThemePartSize(hTheme, NULL, WP_SMALLCLOSEBUTTON, CBS_NORMAL,
-			NULL, TS_TRUE, &closeSize);
+						      nullptr, TS_TRUE, &closeSize);
 		//HRESULT hr = ::GetThemePartSize(hTheme, NULL, WP_MDICLOSEBUTTON, CBS_NORMAL,
 		//	NULL, TS_TRUE, &closeSize);
 		if (!SUCCEEDED(hr)) {
@@ -512,19 +531,19 @@ void Strip::SetTheme() {
 #endif
 }
 
-static bool HideKeyboardCues() {
+static bool HideKeyboardCues() noexcept {
 	BOOL b=FALSE;
 	if (::SystemParametersInfo(SPI_GETKEYBOARDCUES, 0, &b, 0) == 0)
 		return FALSE;
 	return !b;
 }
 
-LRESULT Strip::EditColour(HWND hwnd, HDC hdc) {
+LRESULT Strip::EditColour(HWND hwnd, HDC hdc) noexcept {
 	return ::DefWindowProc(Hwnd(), WM_CTLCOLOREDIT, reinterpret_cast<WPARAM>(hdc),
-		reinterpret_cast<LPARAM>(hwnd));
+			       reinterpret_cast<LPARAM>(hwnd));
 }
 
-LRESULT Strip::CustomDraw(NMHDR *pnmh) {
+LRESULT Strip::CustomDraw(NMHDR *pnmh) noexcept {
 	const int btnStyle = GetWindowStyle(pnmh->hwndFrom);
 	if ((btnStyle & BS_AUTOCHECKBOX) != BS_AUTOCHECKBOX) {
 		return CDRF_DODEFAULT;
@@ -551,25 +570,25 @@ LRESULT Strip::CustomDraw(NMHDR *pnmh) {
 		::FillRect(pcd->hdc, &pcd->rc, hbrFace);
 		::DeleteObject(hbrFace);
 		::DrawThemeBackground(hThemeButton, pcd->hdc, TP_BUTTON, buttonAppearence,
-			&pcd->rc, NULL);
+				      &pcd->rc, nullptr);
 
 		RECT rcButton = pcd->rc;
 		rcButton.bottom--;
 		const HRESULT hr = ::GetThemeBackgroundContentRect(hThemeButton, pcd->hdc, TP_BUTTON,
-			buttonAppearence, &pcd->rc, &rcButton);
+				   buttonAppearence, &pcd->rc, &rcButton);
 		if (!SUCCEEDED(hr)) {
 			return CDRF_DODEFAULT;
 		}
 
 		HBITMAP hBitmap = reinterpret_cast<HBITMAP>(::SendMessage(
-			pnmh->hwndFrom, BM_GETIMAGE, IMAGE_BITMAP, 0));
+					  pnmh->hwndFrom, BM_GETIMAGE, IMAGE_BITMAP, 0));
 
 		// Retrieve the bitmap dimensions
 		BITMAPINFO rbmi = BITMAPINFO();
-		rbmi.bmiHeader.biSize = sizeof (BITMAPINFOHEADER);
-		::GetDIBits(pcd->hdc, hBitmap, 0, 0, NULL, &rbmi, DIB_RGB_COLORS);
+		rbmi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
+		::GetDIBits(pcd->hdc, hBitmap, 0, 0, nullptr, &rbmi, DIB_RGB_COLORS);
 
-		const DWORD colourTransparent = RGB(0xC0,0xC0,0xC0);
+		const DWORD colourTransparent = RGB(0xC0, 0xC0, 0xC0);
 
 		// Offset from button edge to contents.
 		const int xOffset = ((rcButton.right - rcButton.left) - rbmi.bmiHeader.biWidth) / 2 + 1;
@@ -578,8 +597,8 @@ LRESULT Strip::CustomDraw(NMHDR *pnmh) {
 		HDC hdcBM = ::CreateCompatibleDC(NULL);
 		HBITMAP hbmOriginal = SelectBitmap(hdcBM, hBitmap);
 		::TransparentBlt(pcd->hdc, xOffset, yOffset,
-			rbmi.bmiHeader.biWidth, rbmi.bmiHeader.biHeight,
-			hdcBM, 0, 0, rbmi.bmiHeader.biWidth, rbmi.bmiHeader.biHeight, colourTransparent);
+				 rbmi.bmiHeader.biWidth, rbmi.bmiHeader.biHeight,
+				 hdcBM, 0, 0, rbmi.bmiHeader.biWidth, rbmi.bmiHeader.biHeight, colourTransparent);
 		SelectBitmap(hdcBM, hbmOriginal);
 		::DeleteDC(hdcBM);
 
@@ -694,8 +713,8 @@ LRESULT Strip::WndProc(UINT iMessage, WPARAM wParam, LPARAM lParam) {
 			} else if (pnmh->code == TTN_GETDISPINFO) {
 				NMTTDISPINFOW *pnmtdi = reinterpret_cast<LPNMTTDISPINFO>(lParam);
 				const int idButton = static_cast<int>(
-					(pnmtdi->uFlags & TTF_IDISHWND) ?
-					::GetDlgCtrlID(reinterpret_cast<HWND>(pnmtdi->hdr.idFrom)) : pnmtdi->hdr.idFrom);
+							     (pnmtdi->uFlags & TTF_IDISHWND) ?
+							     ::GetDlgCtrlID(reinterpret_cast<HWND>(pnmtdi->hdr.idFrom)) : pnmtdi->hdr.idFrom);
 				for (const SearchOption &so : toggles) {
 					if (so.id == idButton) {
 						GUI::gui_string localised = localiser->Text(so.label);
@@ -722,19 +741,19 @@ void BackgroundStrip::Creation() {
 	Strip::Creation();
 
 	wExplanation = ::CreateWindowEx(0, TEXT("Static"), TEXT(""),
-		WS_CHILD | WS_CLIPSIBLINGS,
-		2, 2, 100, 21,
-		Hwnd(), HmenuID(0), ::GetModuleHandle(NULL), 0);
+					WS_CHILD | WS_CLIPSIBLINGS,
+					2, 2, 100, 21,
+					Hwnd(), HmenuID(0), ::ApplicationInstance(), 0);
 	wExplanation.Show();
 	SetFontHandle(wExplanation, fontText);
 
 	wProgress = ::CreateWindowEx(0, PROGRESS_CLASS, TEXT(""),
-		WS_CHILD | WS_CLIPSIBLINGS | WS_VISIBLE,
-		2, 2, 100, 21,
-		Hwnd(), HmenuID(0), ::GetModuleHandle(NULL), 0);
+				     WS_CHILD | WS_CLIPSIBLINGS | WS_VISIBLE,
+				     2, 2, 100, 21,
+				     Hwnd(), HmenuID(0), ::ApplicationInstance(), 0);
 }
 
-void BackgroundStrip::Destruction() {
+void BackgroundStrip::Destruction() noexcept {
 	Strip::Destruction();
 }
 
@@ -763,14 +782,14 @@ void BackgroundStrip::Size() {
 	rcExplanation.bottom += 1;
 	wExplanation.SetPosition(rcExplanation);
 
-	::InvalidateRect(Hwnd(), NULL, TRUE);
+	::InvalidateRect(Hwnd(), nullptr, TRUE);
 }
 
-bool BackgroundStrip::HasClose() const {
+bool BackgroundStrip::HasClose() const noexcept {
 	return false;
 }
 
-void BackgroundStrip::Focus() {
+void BackgroundStrip::Focus() noexcept {
 	::SetFocus(HwndOf(wExplanation));
 }
 
@@ -805,7 +824,7 @@ void BackgroundStrip::SetProgress(const GUI::gui_string &explanation, size_t siz
 	::SendMessage(HwndOf(wProgress), PBM_SETPOS, progress/scaleProgress, 0);
 }
 
-static const COLORREF colourNoMatch = RGB(0xff,0x66,0x66);
+static const COLORREF colourNoMatch = RGB(0xff, 0x66, 0x66);
 
 void SearchStripBase::Creation() {
 	Strip::Creation();
@@ -813,13 +832,13 @@ void SearchStripBase::Creation() {
 	hbrNoMatch = CreateSolidBrush(colourNoMatch);
 }
 
-void SearchStripBase::Destruction() {
+void SearchStripBase::Destruction() noexcept {
 	::DeleteObject(hbrNoMatch);
-	hbrNoMatch = 0;
+	hbrNoMatch = {};
 	Strip::Destruction();
 }
 
-LRESULT SearchStripBase::NoMatchColour(HDC hdc) {
+LRESULT SearchStripBase::NoMatchColour(HDC hdc) noexcept {
 	SetTextColor(hdc, RGB(0xff, 0xff, 0xff));
 	SetBkColor(hdc, colourNoMatch);
 	return reinterpret_cast<LRESULT>(hbrNoMatch);
@@ -831,9 +850,9 @@ void SearchStrip::Creation() {
 	wStaticFind = CreateText(textFindPrompt);
 
 	wText = CreateWindowEx(WS_EX_CLIENTEDGE, TEXT("Edit"), TEXT(""),
-		WS_CHILD | WS_TABSTOP | WS_CLIPSIBLINGS | ES_AUTOHSCROLL,
-		50, 2, 300, 21,
-		Hwnd(), HmenuID(IDC_INCFINDTEXT), ::GetModuleHandle(NULL), 0);
+			       WS_CHILD | WS_TABSTOP | WS_CLIPSIBLINGS | ES_AUTOHSCROLL,
+			       50, 2, 300, 21,
+			       Hwnd(), HmenuID(IDC_INCFINDTEXT), ::ApplicationInstance(), 0);
 	wText.Show();
 
 	SetFontHandle(wText, fontText);
@@ -844,7 +863,7 @@ void SearchStrip::Creation() {
 	lineHeight = rcButton.Height() + space + 1;
 }
 
-void SearchStrip::Destruction() {
+void SearchStrip::Destruction() noexcept {
 	SearchStripBase::Destruction();
 }
 
@@ -879,14 +898,14 @@ void SearchStrip::Size() {
 
 	wStaticFind.SetPosition(rcText);
 
-	::InvalidateRect(Hwnd(), NULL, TRUE);
+	::InvalidateRect(Hwnd(), nullptr, TRUE);
 }
 
 void SearchStrip::Paint(HDC hDC) {
 	Strip::Paint(hDC);
 }
 
-void SearchStrip::Focus() {
+void SearchStrip::Focus() noexcept {
 	::SetFocus(HwndOf(wText));
 }
 
@@ -927,15 +946,15 @@ bool SearchStrip::Command(WPARAM wParam) {
 	const int control = ControlIDOfWParam(wParam);
 	const int subCommand = static_cast<int>(wParam >> 16);
 	if (((control == IDC_INCFINDTEXT) && (subCommand == EN_CHANGE)) ||
-		(control == IDC_INCFINDBTNOK)) {
+			(control == IDC_INCFINDBTNOK)) {
 		Next(control != IDC_INCFINDBTNOK);
 		return true;
 	}
 	return false;
 }
 
-LRESULT SearchStrip::EditColour(HWND hwnd, HDC hdc) {
-	if (GetDlgItem(static_cast<HWND>(GetID()),IDC_INCFINDTEXT) == hwnd) {
+LRESULT SearchStrip::EditColour(HWND hwnd, HDC hdc) noexcept {
+	if (GetDlgItem(static_cast<HWND>(GetID()), IDC_INCFINDTEXT) == hwnd) {
 		if (pSearcher->FindHasText() && pSearcher->failedfind) {
 			return NoMatchColour(hdc);
 		}
@@ -951,11 +970,11 @@ LRESULT SearchStrip::WndProc(UINT iMessage, WPARAM wParam, LPARAM lParam) {
 	return 0;
 }
 
-LRESULT FindReplaceStrip::EditColour(HWND hwnd, HDC hdc) {
-	if (GetDlgItem(static_cast<HWND>(GetID()),IDFINDWHAT) == ::GetParent(hwnd)) {
+LRESULT FindReplaceStrip::EditColour(HWND hwnd, HDC hdc) noexcept {
+	if (GetDlgItem(static_cast<HWND>(GetID()), IDFINDWHAT) == ::GetParent(hwnd)) {
 		if (pSearcher->FindHasText() &&
-			(incrementalBehaviour != simple) &&
-			pSearcher->failedfind) {
+				(incrementalBehaviour != simple) &&
+				pSearcher->failedfind) {
 			return NoMatchColour(hdc);
 		}
 	}
@@ -983,7 +1002,7 @@ void FindReplaceStrip::NextIncremental(ChangingSource source) {
 	wText.InvalidateAll();
 }
 
-void FindReplaceStrip::SetIncrementalBehaviour(int behaviour) {
+void FindReplaceStrip::SetIncrementalBehaviour(int behaviour) noexcept {
 	incrementalBehaviour = static_cast<FindReplaceStrip::IncrementalBehaviour>(behaviour);
 }
 
@@ -1007,9 +1026,9 @@ void FindStrip::Creation() {
 	wStaticFind = CreateText(textFindPrompt);
 
 	wText = CreateWindowEx(0, TEXT("ComboBox"), TEXT(""),
-		WS_CHILD | WS_TABSTOP | WS_CLIPSIBLINGS | CBS_DROPDOWN | CBS_AUTOHSCROLL,
-		50, 2, 300, 80,
-		Hwnd(), HmenuID(IDFINDWHAT), ::GetModuleHandle(NULL), 0);
+			       WS_CHILD | WS_TABSTOP | WS_CLIPSIBLINGS | CBS_DROPDOWN | CBS_AUTOHSCROLL,
+			       50, 2, 300, 80,
+			       Hwnd(), HmenuID(IDFINDWHAT), ::ApplicationInstance(), 0);
 	SetFontHandle(wText, fontText);
 	wText.Show();
 
@@ -1027,7 +1046,7 @@ void FindStrip::Creation() {
 	wCheckUp = CreateButton(toggles[SearchOption::tUp].label, toggles[SearchOption::tUp].id, true);
 }
 
-void FindStrip::Destruction() {
+void FindStrip::Destruction() noexcept {
 	SearchStripBase::Destruction();
 }
 
@@ -1086,14 +1105,14 @@ void FindStrip::Size() {
 	rcText.bottom = rcArea.bottom;
 	wStaticFind.SetPosition(rcText);
 
-	::InvalidateRect(Hwnd(), NULL, TRUE);
+	::InvalidateRect(Hwnd(), nullptr, TRUE);
 }
 
 void FindStrip::Paint(HDC hDC) {
 	Strip::Paint(hDC);
 }
 
-void FindStrip::Focus() {
+void FindStrip::Focus() noexcept {
 	::SetFocus(HwndOf(wText));
 }
 
@@ -1122,14 +1141,14 @@ bool FindStrip::KeyDown(WPARAM key) {
 
 void FindStrip::Next(bool markAll, bool invertDirection) {
 	pSearcher->SetFind(ControlText(wText).c_str());
-	if (markAll){
+	if (markAll) {
 		pSearcher->MarkAll(Searcher::markWithBookMarks);
 	}
 	const bool found = pSearcher->FindNext(pSearcher->reverseFind ^ invertDirection) >= 0;
 	if (pSearcher->ShouldClose(found)) {
 		Close();
 	} else {
- 		SetComboFromMemory(wText, pSearcher->memFinds);
+		SetComboFromMemory(wText, pSearcher->memFinds);
 		SetComboText(wText, pSearcher->findWhat, ComboSelection::atEnd);
 	}
 }
@@ -1189,7 +1208,7 @@ bool FindStrip::Command(WPARAM wParam) {
 	return false;
 }
 
-void FindStrip::CheckButtons() {
+void FindStrip::CheckButtons() noexcept {
 	entered++;
 	CheckButton(wCheckWord, pSearcher->wholeWord);
 	CheckButton(wCheckCase, pSearcher->matchCase);
@@ -1219,9 +1238,9 @@ void ReplaceStrip::Creation() {
 	wStaticFind = CreateText(textFindPrompt);
 
 	wText = CreateWindowEx(0, TEXT("ComboBox"), TEXT(""),
-		WS_CHILD | WS_TABSTOP | WS_CLIPSIBLINGS | CBS_DROPDOWN | CBS_AUTOHSCROLL,
-		50, 2, 300, 80,
-		Hwnd(), HmenuID(IDFINDWHAT), ::GetModuleHandle(NULL), 0);
+			       WS_CHILD | WS_TABSTOP | WS_CLIPSIBLINGS | CBS_DROPDOWN | CBS_AUTOHSCROLL,
+			       50, 2, 300, 80,
+			       Hwnd(), HmenuID(IDFINDWHAT), ::ApplicationInstance(), 0);
 	SetFontHandle(wText, fontText);
 	wText.Show();
 
@@ -1231,9 +1250,9 @@ void ReplaceStrip::Creation() {
 	wStaticReplace = CreateText(textReplacePrompt);
 
 	wReplace = CreateWindowEx(0, TEXT("ComboBox"), TEXT(""),
-		WS_CHILD | WS_TABSTOP | CBS_DROPDOWN | CBS_AUTOHSCROLL,
-		50, 2, 300, 80,
-		Hwnd(), HmenuID(IDREPLACEWITH), ::GetModuleHandle(NULL), 0);
+				  WS_CHILD | WS_TABSTOP | CBS_DROPDOWN | CBS_AUTOHSCROLL,
+				  50, 2, 300, 80,
+				  Hwnd(), HmenuID(IDREPLACEWITH), ::ApplicationInstance(), 0);
 	SetFontHandle(wReplace, fontText);
 	wReplace.Show();
 
@@ -1252,11 +1271,11 @@ void ReplaceStrip::Creation() {
 	wCheckWrap = CreateButton(toggles[SearchOption::tWrap].label, toggles[SearchOption::tWrap].id, true);
 }
 
-void ReplaceStrip::Destruction() {
+void ReplaceStrip::Destruction() noexcept {
 	SearchStripBase::Destruction();
 }
 
-int ReplaceStrip::Lines() const {
+int ReplaceStrip::Lines() const noexcept {
 	return 2;
 }
 
@@ -1343,18 +1362,18 @@ void ReplaceStrip::Size() {
 	rcStatic.top = rcLine.top + 3;
 	wStaticReplace.SetPosition(rcStatic);
 
-	::InvalidateRect(Hwnd(), NULL, TRUE);
+	::InvalidateRect(Hwnd(), nullptr, TRUE);
 }
 
 void ReplaceStrip::Paint(HDC hDC) {
 	Strip::Paint(hDC);
 }
 
-void ReplaceStrip::Focus() {
+void ReplaceStrip::Focus() noexcept {
 	::SetFocus(HwndOf(wText));
 }
 
-static bool IsSameOrChild(const GUI::Window &wParent, HWND wChild) {
+static bool IsSameOrChild(const GUI::Window &wParent, HWND wChild) noexcept {
 	HWND hwnd = HwndOf(wParent);
 	return (wChild == hwnd) || IsChild(hwnd, wChild);
 }
@@ -1411,7 +1430,7 @@ void ReplaceStrip::HandleReplaceCommand(int cmd, bool reverseFind) {
 	SetComboText(wText, pSearcher->findWhat, ComboSelection::atEnd);
 	if (cmd != IDOK) {
 		pSearcher->SetReplace(ControlText(wReplace).c_str());
- 		SetComboFromMemory(wReplace, pSearcher->memReplaces);
+		SetComboFromMemory(wReplace, pSearcher->memReplaces);
 		SetComboText(wReplace, pSearcher->replaceWhat, ComboSelection::atEnd);
 	}
 	//int replacements = 0;
@@ -1481,7 +1500,7 @@ bool ReplaceStrip::Command(WPARAM wParam) {
 	return false;
 }
 
-void ReplaceStrip::CheckButtons() {
+void ReplaceStrip::CheckButtons() noexcept {
 	entered++;
 	CheckButton(wCheckWord, pSearcher->wholeWord);
 	CheckButton(wCheckCase, pSearcher->matchCase);
@@ -1513,9 +1532,9 @@ void UserStrip::Creation() {
 	Strip::Creation();
 	// Combo boxes automatically size to a reasonable height so create a temporary and measure
 	HWND wComboTest = ::CreateWindowEx(0, TEXT("ComboBox"), TEXT("Aby"),
-		WS_CHILD | WS_TABSTOP | WS_CLIPSIBLINGS | CBS_DROPDOWN | CBS_AUTOHSCROLL,
-		50, 2, 300, 80,
-		Hwnd(), 0, ::GetModuleHandle(NULL), 0);
+					   WS_CHILD | WS_TABSTOP | WS_CLIPSIBLINGS | CBS_DROPDOWN | CBS_AUTOHSCROLL,
+					   50, 2, 300, 80,
+					   Hwnd(), 0, ::ApplicationInstance(), 0);
 	SetWindowFont(wComboTest, fontText, 0);
 	RECT rc;
 	::GetWindowRect(wComboTest, &rc);
@@ -1523,7 +1542,7 @@ void UserStrip::Creation() {
 	lineHeight = rc.bottom - rc.top + 3;
 }
 
-void UserStrip::Destruction() {
+void UserStrip::Destruction() noexcept {
 	psd.reset();
 	Strip::Destruction();
 }
@@ -1592,14 +1611,14 @@ void UserStrip::Size() {
 		top += lineHeight;
 	}
 
-	::InvalidateRect(Hwnd(), NULL, TRUE);
+	::InvalidateRect(Hwnd(), nullptr, TRUE);
 }
 
-bool UserStrip::HasClose() const {
+bool UserStrip::HasClose() const noexcept {
 	return psd && psd->hasClose;
 }
 
-void UserStrip::Focus() {
+void UserStrip::Focus() noexcept {
 	for (const std::vector<UserControl> &line : psd->controls) {
 		for (const UserControl &ctl : line) {
 			if (ctl.controlType != UserControl::ucStatic) {
@@ -1632,23 +1651,23 @@ bool UserStrip::KeyDown(WPARAM key) {
 	return false;
 }
 
-static StripCommand NotificationToStripCommand(int notification) {
+static StripCommand NotificationToStripCommand(int notification) noexcept {
 	switch (notification) {
-		case BN_CLICKED:
-			return scClicked;
-		case EN_CHANGE:
-		case CBN_EDITCHANGE:
-			return scChange;
-		case EN_UPDATE:
-			return scUnknown;
-		case CBN_SETFOCUS:
-		case EN_SETFOCUS:
-			return scFocusIn;
-		case CBN_KILLFOCUS:
-		case EN_KILLFOCUS:
-			return scFocusOut;
-		default:
-			return scUnknown;
+	case BN_CLICKED:
+		return scClicked;
+	case EN_CHANGE:
+	case CBN_EDITCHANGE:
+		return scChange;
+	case EN_UPDATE:
+		return scUnknown;
+	case CBN_SETFOCUS:
+	case EN_SETFOCUS:
+		return scFocusIn;
+	case CBN_KILLFOCUS:
+	case EN_KILLFOCUS:
+		return scFocusOut;
+	default:
+		return scUnknown;
 	}
 }
 
@@ -1673,7 +1692,7 @@ LRESULT UserStrip::WndProc(UINT iMessage, WPARAM wParam, LPARAM lParam) {
 	return 0;
 }
 
-int UserStrip::Lines() const {
+int UserStrip::Lines() const noexcept {
 	return psd ? static_cast<int>(psd->controls.size()) : 1;
 }
 
@@ -1700,38 +1719,38 @@ void UserStrip::SetDescription(const char *description) {
 				ctl.widthDesired = 100;
 				ctl.fixedWidth = false;
 				ctl.w = ::CreateWindowEx(WS_EX_CLIENTEDGE, TEXT("Edit"), ctl.text.c_str(),
-					WS_CHILD | WS_TABSTOP | WS_CLIPSIBLINGS | ES_AUTOHSCROLL,
-					left, top, ctl.widthDesired, lineHeight - 3,
-					Hwnd(), HmenuID(controlID), ::GetModuleHandle(NULL), 0);
+							 WS_CHILD | WS_TABSTOP | WS_CLIPSIBLINGS | ES_AUTOHSCROLL,
+							 left, top, ctl.widthDesired, lineHeight - 3,
+							 Hwnd(), HmenuID(controlID), ::ApplicationInstance(), 0);
 				break;
 
 			case UserControl::ucCombo:
 				ctl.widthDesired = 100;
 				ctl.fixedWidth = false;
 				ctl.w = ::CreateWindowEx(WS_EX_CLIENTEDGE, TEXT("ComboBox"), ctl.text.c_str(),
-					WS_CHILD | WS_TABSTOP | WS_CLIPSIBLINGS | CBS_DROPDOWN | CBS_AUTOHSCROLL | WS_VSCROLL,
-					left, top, ctl.widthDesired, 180,
-					Hwnd(), HmenuID(controlID), ::GetModuleHandle(NULL), 0);
+							 WS_CHILD | WS_TABSTOP | WS_CLIPSIBLINGS | CBS_DROPDOWN | CBS_AUTOHSCROLL | WS_VSCROLL,
+							 left, top, ctl.widthDesired, 180,
+							 Hwnd(), HmenuID(controlID), ::ApplicationInstance(), 0);
 				break;
 
 			case UserControl::ucButton:
 			case UserControl::ucDefaultButton:
 				ctl.widthDesired = WidthText(fontText, ctl.text.c_str()) +
-					2 * ::GetSystemMetrics(SM_CXEDGE) +
-					2 * WidthText(fontText, TEXT(" "));
+						   2 * ::GetSystemMetrics(SM_CXEDGE) +
+						   2 * WidthText(fontText, TEXT(" "));
 				ctl.w = ::CreateWindowEx(0, TEXT("Button"), ctl.text.c_str(),
-					WS_CHILD | WS_TABSTOP | WS_CLIPSIBLINGS |
-					((ctl.controlType == UserControl::ucDefaultButton) ? BS_DEFPUSHBUTTON : BS_PUSHBUTTON),
-					left, top, ctl.widthDesired, lineHeight-1,
-					Hwnd(), HmenuID(controlID), ::GetModuleHandle(NULL), 0);
+							 WS_CHILD | WS_TABSTOP | WS_CLIPSIBLINGS |
+							 ((ctl.controlType == UserControl::ucDefaultButton) ? BS_DEFPUSHBUTTON : BS_PUSHBUTTON),
+							 left, top, ctl.widthDesired, lineHeight-1,
+							 Hwnd(), HmenuID(controlID), ::ApplicationInstance(), 0);
 				break;
 
 			default:
 				ctl.widthDesired = WidthText(fontText, ctl.text.c_str());
 				ctl.w = ::CreateWindowEx(0, TEXT("Static"), ctl.text.c_str(),
-					WS_CHILD | WS_CLIPSIBLINGS | ES_RIGHT,
-					left, top, ctl.widthDesired, lineHeight - 5,
-					Hwnd(), HmenuID(controlID), ::GetModuleHandle(NULL), 0);
+							 WS_CHILD | WS_CLIPSIBLINGS | ES_RIGHT,
+							 left, top, ctl.widthDesired, lineHeight - 5,
+							 Hwnd(), HmenuID(controlID), ::ApplicationInstance(), 0);
 				break;
 			}
 			ctl.w.Show();
@@ -1747,11 +1766,11 @@ void UserStrip::SetDescription(const char *description) {
 	Focus();
 }
 
-void UserStrip::SetExtender(Extension *extender_) {
+void UserStrip::SetExtender(Extension *extender_) noexcept {
 	extender = extender_;
 }
 
-void UserStrip::SetSciTE(SciTEWin *pSciTEWin_) {
+void UserStrip::SetSciTE(SciTEWin *pSciTEWin_) noexcept {
 	pSciTEWin = pSciTEWin_;
 }
 
@@ -1776,7 +1795,7 @@ void UserStrip::SetList(int control, const char *value) {
 			HWND combo = HwndOf(ctl->w);
 			ComboBox_ResetContent(combo);
 			for (const GUI::gui_string &gs : listValues) {
-				ComboBox_AddString(combo, gs.c_str());
+				ComboBoxAppend(combo, gs);
 			}
 		}
 	}
