@@ -14,18 +14,19 @@
 class WinMutex : public Mutex {
 private:
 	CRITICAL_SECTION cs;
-	void Lock() override { ::EnterCriticalSection(&cs); }
-	void Unlock() override { ::LeaveCriticalSection(&cs); }
+	void Lock() noexcept override { ::EnterCriticalSection(&cs); }
+	void Unlock() noexcept override { ::LeaveCriticalSection(&cs); }
 	WinMutex() { ::InitializeCriticalSection(&cs); }
 	// Deleted so WinMutex objects can not be copied.
 	WinMutex(const WinMutex &) = delete;
 	WinMutex(WinMutex &&) = delete;
 	void operator=(const WinMutex &) = delete;
 	void operator=(WinMutex &&) = delete;
-	virtual ~WinMutex() { ::DeleteCriticalSection(&cs); }
 	friend class Mutex;
+public:
+	virtual ~WinMutex() { ::DeleteCriticalSection(&cs); }
 };
 
 Mutex *Mutex::Create() {
-   return new WinMutex();
+	return new WinMutex();
 }
